"""
Auto-Coder SDK 核心封装类

提供统一的查询接口，处理同步和异步调用。
"""

from typing import AsyncIterator, Optional, Dict, Any, Iterator, List
import asyncio
import os
import time
from concurrent.futures import ThreadPoolExecutor
import json

# Rich 渲染相关导入
from rich.console import Console
from rich.panel import Panel
from rich.markdown import Markdown
from rich.syntax import Syntax
from rich.progress import Progress, SpinnerColumn, TextColumn

from ..models.options import AutoCodeOptions, KnowledgeBaseConfig, CustomRuleConfig
from ..models.responses import StreamEvent, CodeModificationResult
from ..exceptions import BridgeError
from .bridge import AutoCoderBridge
from autocoder.common.wrap_llm_hint.utils import (
    extract_content_from_text,
    has_hint_in_text,
)


class AutoCoderCore:
    """AutoCoder核心封装类"""

    def __init__(self, options: AutoCodeOptions):
        """
        初始化AutoCoderCore

        Args:
            options: 配置选项
        """
        self.options = options
        cwd_str = str(options.cwd) if options.cwd is not None else os.getcwd()
        self.bridge = AutoCoderBridge(cwd_str, options)
        self._executor = ThreadPoolExecutor(max_workers=1)
        self._console = Console()
        self._baseline_custom_knowledge_bases: List[KnowledgeBaseConfig] = [
            KnowledgeBaseConfig.from_dict(kb.to_dict())
            for kb in options.custom_knowledge_bases
        ]
        self._baseline_custom_rules: List[CustomRuleConfig] = [
            CustomRuleConfig.from_dict(rule.to_dict()) for rule in options.custom_rules
        ]

        # 用于累计TokenUsageEvent数据
        self._accumulated_token_usage = {
            "model_name": "",
            "input_tokens": 0,
            "output_tokens": 0,
            "input_cost": 0.0,
            "output_cost": 0.0,
        }

    @staticmethod
    def _clone_knowledge_base(kb: KnowledgeBaseConfig) -> KnowledgeBaseConfig:
        return KnowledgeBaseConfig.from_dict(kb.to_dict())

    def register_knowledge_base(
        self, knowledge_base: KnowledgeBaseConfig, persist: Optional[bool] = None
    ) -> None:
        """Register a custom knowledge base for current core instance."""
        normalized_name = knowledge_base.name.strip()
        if not normalized_name:
            raise ValueError("Knowledge base name cannot be empty")

        normalized = KnowledgeBaseConfig(
            name=normalized_name,
            type=knowledge_base.type,
            path=knowledge_base.path,
            base_url=knowledge_base.base_url,
            api_key=knowledge_base.api_key,
            description=knowledge_base.description,
            persist=knowledge_base.persist if persist is None else persist,
        )

        for existing_kb in self.options.custom_knowledge_bases:
            if existing_kb.name == normalized.name:
                raise ValueError(
                    f"Knowledge base name conflict in options: {normalized.name}"
                )

        candidate_kbs = [*self.options.custom_knowledge_bases, normalized]
        validated_options = self.options.copy(custom_knowledge_bases=candidate_kbs)
        self.options.custom_knowledge_bases = validated_options.custom_knowledge_bases

    def register_knowledge_bases(
        self,
        knowledge_bases: List[KnowledgeBaseConfig],
        persist: Optional[bool] = None,
    ) -> None:
        """Batch register custom knowledge bases for current core instance."""
        for knowledge_base in knowledge_bases:
            self.register_knowledge_base(knowledge_base, persist=persist)

    def clear_runtime_knowledge_bases(self) -> None:
        """Clear runtime-registered custom knowledge bases."""
        self.options.custom_knowledge_bases = [
            self._clone_knowledge_base(kb)
            for kb in self._baseline_custom_knowledge_bases
        ]

    @staticmethod
    def _clone_custom_rule(rule: CustomRuleConfig) -> CustomRuleConfig:
        return CustomRuleConfig.from_dict(rule.to_dict())

    def register_rule(
        self, rule: CustomRuleConfig, persist: Optional[bool] = None
    ) -> None:
        """Register a custom rule for current core instance."""
        normalized_name = rule.name.strip()
        if not normalized_name:
            raise ValueError("Rule name cannot be empty")

        normalized = CustomRuleConfig(
            name=normalized_name,
            content=rule.content,
            persist=rule.persist if persist is None else persist,
            description=rule.description,
            globs=rule.globs,
            always_apply=rule.always_apply,
        )

        for existing_rule in self.options.custom_rules:
            if existing_rule.name == normalized.name:
                raise ValueError(f"Rule name conflict in options: {normalized.name}")

        candidate_rules = [*self.options.custom_rules, normalized]
        validated_options = self.options.copy(custom_rules=candidate_rules)
        self.options.custom_rules = validated_options.custom_rules

    def register_rules(
        self,
        rules: List[CustomRuleConfig],
        persist: Optional[bool] = None,
    ) -> None:
        """Batch register custom rules for current core instance."""
        for rule in rules:
            self.register_rule(rule, persist=persist)

    def clear_runtime_rules(self) -> None:
        """Clear runtime-registered custom rules."""
        self.options.custom_rules = [
            self._clone_custom_rule(rule) for rule in self._baseline_custom_rules
        ]

    def _build_extra_args(
        self, extra_args: Optional[Dict[str, Any]] = None
    ) -> Dict[str, Any]:
        final_extra_args: Dict[str, Any] = dict(extra_args or {})
        custom_knowledge_bases = [
            kb.to_dict() for kb in self.options.custom_knowledge_bases
        ]
        if custom_knowledge_bases:
            final_extra_args["custom_knowledge_bases"] = custom_knowledge_bases
        custom_rules = [rule.to_dict() for rule in self.options.custom_rules]
        if custom_rules:
            final_extra_args["custom_rules"] = custom_rules
        return final_extra_args

    def _render_stream_event(
        self, event: StreamEvent, show_terminal: bool = True
    ) -> None:
        """
        渲染流式事件到终端

        Args:
            event: 流式事件
            show_terminal: 是否显示到终端
        """
        if not show_terminal:
            return

        # 如果输出格式是 JSON 相关，输出 JSON 格式
        if self.options.output_format in ["json", "stream-json"]:
            self._render_json_event(event)
            return

        # 原有的文本渲染逻辑保持不变
        try:
            # 处理新的事件类型（动态检查以避免导入依赖）
            event_class_name = type(event).__name__

            # 处理 TokenUsageEvent 和 WindowLengthChangeEvent
            if "TokenUsageEvent" in event_class_name:
                usage = getattr(event, "usage", None)
                if usage:
                    self._process_token_usage_event(usage)
                return

            elif "WindowLengthChangeEvent" in event_class_name:
                tokens_used = getattr(event, "tokens_used", 0)
                if tokens_used > 0:
                    self._console.print(f"[dim]当前会话总 tokens: {tokens_used}[/dim]")
                return

            elif "LLMThinkingEvent" in event_class_name:
                text = getattr(event, "text", "")
                if text.strip():
                    self._console.print(f"[grey50]{text}[/grey50]", end="")
                return

            elif "LLMOutputEvent" in event_class_name:
                text = getattr(event, "text", "")
                if text.strip():
                    self._console.print(text, end="")
                return

            elif "ToolCallEvent" in event_class_name:
                # 跳过 AttemptCompletionTool 的工具调用显示
                tool = getattr(event, "tool", None)
                if tool and "AttemptCompletionTool" in type(tool).__name__:
                    return

                tool_name = type(tool).__name__ if tool else "Unknown Tool"

                # 特殊处理 ConversationMessageIds 工具 - 只显示 compacting 消息
                if tool and tool_name in [
                    "ConversationMessageIdsWriteTool",
                    "ConversationMessageIdsReadTool",
                ]:
                    # 使用国际化的 compacting 消息
                    try:
                        from autocoder.common.auto_coder_lang import get_system_language

                        lang = get_system_language()
                        compacting_message = (
                            "compacting..." if lang == "en" else "压缩中..."
                        )
                    except:
                        compacting_message = "compacting..."

                    self._console.print(
                        Panel(
                            f"[dim]{compacting_message}[/dim]",
                            title="🗜️ Compacting",
                            border_style="yellow",
                            title_align="left",
                        )
                    )
                    return

                try:
                    # 尝试使用 get_tool_display_message 函数
                    from autocoder.common.v2.agent.agentic_edit_types import (
                        get_tool_display_message,
                    )

                    display_content = get_tool_display_message(tool)
                except:
                    # 如果导入失败，使用简单的显示
                    display_content = f"Tool: {tool_name}"
                    if hasattr(tool, "__dict__"):
                        for key, value in tool.__dict__.items():
                            if not key.startswith("_"):
                                display_content += f"\n{key}: {value}"

                self._console.print(
                    Panel(
                        display_content,
                        title=f"🛠️ Action: {tool_name}",
                        border_style="blue",
                        title_align="left",
                    )
                )
                return

            elif "ToolResultEvent" in event_class_name:
                # 跳过 AttemptCompletionTool 和 PlanModeRespondTool 的结果显示
                tool_name = getattr(event, "tool_name", "Unknown")
                if tool_name in ["AttemptCompletionTool", "PlanModeRespondTool"]:
                    return

                # 跳过 ConversationMessageIds 工具的结果显示 - 已由 compacting 消息处理
                if tool_name in [
                    "ConversationMessageIdsWriteTool",
                    "ConversationMessageIdsReadTool",
                ]:
                    return

                result = getattr(event, "result", None)
                if result:
                    success = getattr(result, "success", True)
                    message = getattr(result, "message", "")
                    content = getattr(result, "content", None)

                    title = (
                        f"✅ Tool Result: {tool_name}"
                        if success
                        else f"❌ Tool Result: {tool_name}"
                    )
                    border_style = "green" if success else "red"

                    base_content = (
                        f"[bold]Status:[/bold] {'Success' if success else 'Failure'}\n"
                    )
                    base_content += f"[bold]Message:[/bold] {message}\n"

                    # 处理内容显示
                    if content is not None:
                        formatted_content = self._format_tool_result_content(
                            content, tool_name
                        )
                        if isinstance(formatted_content, Syntax):
                            self._console.print(
                                Panel(
                                    base_content,
                                    title=title,
                                    border_style=border_style,
                                    title_align="left",
                                )
                            )
                            self._console.print(formatted_content)
                        else:
                            base_content += f"\n{formatted_content}"
                            self._console.print(
                                Panel(
                                    base_content,
                                    title=title,
                                    border_style=border_style,
                                    title_align="left",
                                )
                            )
                    else:
                        self._console.print(
                            Panel(
                                base_content,
                                title=title,
                                border_style=border_style,
                                title_align="left",
                            )
                        )
                return

            elif "CompletionEvent" in event_class_name:
                completion = getattr(event, "completion", None)
                if completion:
                    result = getattr(
                        completion, "result", "Task completed successfully"
                    )
                    command = getattr(completion, "command", None)

                    self._console.print(
                        Panel(
                            Markdown(result),
                            title="🏁 Task Completion",
                            border_style="green",
                            title_align="left",
                        )
                    )
                    if command:
                        self._console.print(
                            f"[dim]Suggested command:[/dim] [bold cyan]{command}[/]"
                        )
                return

            elif "PlanModeRespondEvent" in event_class_name:
                completion = getattr(event, "completion", None)
                if completion:
                    response = getattr(completion, "response", "Plan completed")
                    self._console.print(
                        Panel(
                            Markdown(response),
                            title="🏁 Plan Completion",
                            border_style="green",
                            title_align="left",
                        )
                    )
                return

            elif "ErrorEvent" in event_class_name:
                message = getattr(event, "message", "Unknown error")
                self._console.print(
                    Panel(
                        f"[bold red]Error:[/bold red] {message}",
                        title="🔥 Error",
                        border_style="red",
                        title_align="left",
                    )
                )
                return

            elif "ConversationIdEvent" in event_class_name:
                conversation_id = getattr(event, "conversation_id", "")
                if conversation_id:
                    self._console.print(
                        f"[dim]Conversation ID: {conversation_id}[/dim]"
                    )
                return

            elif "PreCommitEvent" in event_class_name:
                # event.commit_result 一定存在
                commit_result = event.commit_result
                success = (
                    commit_result.success
                    if hasattr(commit_result, "success")
                    else False
                )
                message = (
                    commit_result.message
                    if hasattr(commit_result, "message")
                    else "Pre-commit operation completed"
                )

                title = "✅ Pre-Commit Success" if success else "❌ Pre-Commit Failed"
                border_style = "yellow" if success else "red"

                self._console.print(
                    Panel(
                        f"[bold]Status:[/bold] {'Success' if success else 'Failed'}\n[bold]Message:[/bold] {message}",
                        title=title,
                        border_style=border_style,
                        title_align="left",
                    )
                )
                return

            elif "CommitEvent" in event_class_name:
                # event.commit_result 一定存在
                commit_result = event.commit_result
                success = (
                    commit_result.success
                    if hasattr(commit_result, "success")
                    else False
                )
                message = (
                    commit_result.message
                    if hasattr(commit_result, "message")
                    else "Commit operation completed"
                )
                commit_hash = (
                    commit_result.commit_hash
                    if hasattr(commit_result, "commit_hash")
                    else ""
                )

                title = "✅ Commit Success" if success else "❌ Commit Failed"
                border_style = "green" if success else "red"

                content = f"[bold]Status:[/bold] {'Success' if success else 'Failed'}\n[bold]Message:[/bold] {message}"
                if commit_hash:
                    content += f"\n[bold]Commit Hash:[/bold] {commit_hash}"

                self._console.print(
                    Panel(
                        content,
                        title=title,
                        border_style=border_style,
                        title_align="left",
                    )
                )
                return

            elif "PullRequestEvent" in event_class_name:
                # event.pull_request_result 一定存在
                pr_result = event.pull_request_result
                success = pr_result.success if hasattr(pr_result, "success") else False
                pr_number = (
                    pr_result.pr_number if hasattr(pr_result, "pr_number") else None
                )
                pr_url = pr_result.pr_url if hasattr(pr_result, "pr_url") else ""
                error_message = (
                    pr_result.error_message
                    if hasattr(pr_result, "error_message")
                    else ""
                )
                platform = (
                    pr_result.platform if hasattr(pr_result, "platform") else "Unknown"
                )

                title = (
                    "✅ Pull Request Created" if success else "❌ Pull Request Failed"
                )
                border_style = "cyan" if success else "red"

                if success:
                    content = f"[bold]Status:[/bold] Success\n[bold]Platform:[/bold] {platform}"
                    if pr_number:
                        content += f"\n[bold]PR Number:[/bold] #{pr_number}"
                    if pr_url:
                        content += f"\n[bold]PR URL:[/bold] {pr_url}"
                else:
                    content = f"[bold]Status:[/bold] Failed\n[bold]Platform:[/bold] {platform}"
                    if error_message:
                        content += f"\n[bold]Error:[/bold] {error_message}"

                self._console.print(
                    Panel(
                        content,
                        title=title,
                        border_style=border_style,
                        title_align="left",
                    )
                )
                return

            # 处理旧格式的事件类型
            if event.event_type == "start":
                project_name = os.path.basename(
                    os.path.abspath(self.options.cwd or os.getcwd())
                )
                self._console.rule(f"[bold cyan]Starting Auto-Coder: {project_name}[/]")
                query = event.data.get("query", "")
                if query:
                    self._console.print(
                        Panel(
                            f"[bold]Query:[/bold]\n{query}",
                            title="🎯 Objective",
                            border_style="blue",
                        )
                    )

            elif event.event_type == "llm_thinking":
                text = event.data.get("text", "")
                if text.strip():
                    self._console.print(f"[grey50]{text}[/grey50]", end="")

            elif event.event_type == "llm_output":
                text = event.data.get("text", "")
                if text.strip():
                    self._console.print(text, end="")

            elif event.event_type == "tool_call":
                tool_name = event.data.get("tool_name", "Unknown Tool")

                # 特殊处理 ConversationMessageIds 工具 - 只显示 compacting 消息
                if tool_name in [
                    "ConversationMessageIdsWriteTool",
                    "ConversationMessageIdsReadTool",
                ]:
                    # 使用国际化的 compacting 消息
                    try:
                        from autocoder.common.auto_coder_lang import get_system_language

                        lang = get_system_language()
                        compacting_message = (
                            "compacting..." if lang == "en" else "压缩中..."
                        )
                    except:
                        compacting_message = "compacting..."

                    self._console.print(
                        Panel(
                            f"[dim]{compacting_message}[/dim]",
                            title="🗜️ Compacting",
                            border_style="yellow",
                            title_align="left",
                        )
                    )
                else:
                    tool_args = event.data.get("args", {})
                    display_content = self._format_tool_display(tool_name, tool_args)
                    self._console.print(
                        Panel(
                            display_content,
                            title=f"🛠️ Action: {tool_name}",
                            border_style="blue",
                            title_align="left",
                        )
                    )

            elif event.event_type == "tool_result":
                tool_name = event.data.get("tool_name", "Unknown Tool")

                # 跳过 ConversationMessageIds 工具的结果显示 - 已由 compacting 消息处理
                if tool_name in [
                    "ConversationMessageIdsWriteTool",
                    "ConversationMessageIdsReadTool",
                ]:
                    return

                success = event.data.get("success", True)
                message = event.data.get("message", "")
                content = event.data.get("content")

                title = (
                    f"✅ Tool Result: {tool_name}"
                    if success
                    else f"❌ Tool Result: {tool_name}"
                )
                border_style = "green" if success else "red"

                base_content = (
                    f"[bold]Status:[/bold] {'Success' if success else 'Failure'}\n"
                )
                base_content += f"[bold]Message:[/bold] {message}\n"

                # 处理内容显示
                if content is not None:
                    formatted_content = self._format_tool_result_content(
                        content, tool_name
                    )
                    if isinstance(formatted_content, Syntax):
                        self._console.print(
                            Panel(
                                base_content,
                                title=title,
                                border_style=border_style,
                                title_align="left",
                            )
                        )
                        self._console.print(formatted_content)
                    else:
                        base_content += f"\n{formatted_content}"
                        self._console.print(
                            Panel(
                                base_content,
                                title=title,
                                border_style=border_style,
                                title_align="left",
                            )
                        )
                else:
                    self._console.print(
                        Panel(
                            base_content,
                            title=title,
                            border_style=border_style,
                            title_align="left",
                        )
                    )

            elif event.event_type == "completion":
                result = event.data.get("result", "Task completed successfully")
                self._console.print(
                    Panel(
                        Markdown(result),
                        title="🏁 Task Completion",
                        border_style="green",
                        title_align="left",
                    )
                )

            elif event.event_type == "plan_mode_respond":
                result = event.data.get("result", "Plan completed")
                self._console.print(
                    Panel(
                        Markdown(result),
                        title="🏁 Plan Completion",
                        border_style="green",
                        title_align="left",
                    )
                )

            elif event.event_type == "token_usage":
                usage = event.data.get("usage")
                if usage:
                    self._process_token_usage_event(usage)

            elif event.event_type == "window_change":
                tokens_used = event.data.get("tokens_used", 0)
                if tokens_used > 0:
                    self._console.print(f"[dim]当前会话总 tokens: {tokens_used}[/dim]")

            elif event.event_type == "conversation_id":
                conversation_id = event.data.get("conversation_id", "")
                if conversation_id:
                    self._console.print(
                        f"[dim]Conversation ID: {conversation_id}[/dim]"
                    )

            elif event.event_type == "content":
                content = event.data.get("content", "")
                if content.strip():
                    # 检查是否是思考过程（通常包含特定标记）
                    if any(
                        marker in content.lower()
                        for marker in ["thinking", "analyzing", "考虑", "分析"]
                    ):
                        self._console.print(f"[grey50]{content}[/grey50]", end="")
                    else:
                        self._console.print(content, end="")

            elif event.event_type == "file_modified":
                files = event.data.get("files", [])
                if files:
                    files_str = "\n".join([f"  - {f}" for f in files])
                    self._console.print(
                        Panel(
                            f"[bold]Modified Files:[/bold]\n{files_str}",
                            title="📝 File Changes",
                            border_style="yellow",
                            title_align="left",
                        )
                    )

            elif event.event_type == "file_created":
                files = event.data.get("files", [])
                if files:
                    files_str = "\n".join([f"  - {f}" for f in files])
                    self._console.print(
                        Panel(
                            f"[bold]Created Files:[/bold]\n{files_str}",
                            title="📄 New Files",
                            border_style="green",
                            title_align="left",
                        )
                    )

            elif event.event_type == "file_deleted":
                files = event.data.get("files", [])
                if files:
                    files_str = "\n".join([f"  - {f}" for f in files])
                    self._console.print(
                        Panel(
                            f"[bold]Deleted Files:[/bold]\n{files_str}",
                            title="🗑️ Removed Files",
                            border_style="red",
                            title_align="left",
                        )
                    )

            elif event.event_type == "end":
                status = event.data.get("status", "completed")
                if status == "completed":
                    self._console.rule(
                        "[bold green]Auto-Coder Finished Successfully[/]"
                    )
                else:
                    self._console.rule(f"[bold yellow]Auto-Coder Finished: {status}[/]")

            elif event.event_type == "error":
                error = event.data.get("error", "Unknown error")
                error_type = event.data.get("error_type", "Error")
                self._console.print(
                    Panel(
                        f"[bold red]Error Type:[/bold red] {error_type}\n[bold red]Message:[/bold red] {error}",
                        title="🔥 Error",
                        border_style="red",
                        title_align="left",
                    )
                )

        except Exception as e:
            # 渲染错误不应该影响主流程
            self._console.print(f"[dim red]Render error: {str(e)}[/dim red]")

    def _render_json_event(self, event: StreamEvent) -> None:
        """
        以 JSON 格式渲染流式事件

        Args:
            event: 流式事件
        """
        try:
            # 构建事件的 JSON 表示
            event_json = self._convert_event_to_json(event)
            print(json.dumps(event_json, ensure_ascii=False), flush=True)
        except Exception as e:
            # 如果 JSON 序列化失败，输出错误事件
            error_json = {
                "event_type": "json_render_error",
                "data": {
                    "error": str(e),
                    "original_event_type": getattr(event, "event_type", "unknown"),
                },
                "timestamp": time.time(),
            }
            print(json.dumps(error_json, ensure_ascii=False), flush=True)

    def _convert_event_to_json(self, event: StreamEvent) -> dict:
        """
        将流式事件转换为 JSON 字典

        Args:
            event: 流式事件

        Returns:
            dict: JSON 字典表示
        """
        # 处理新格式的事件类型（动态检查以避免导入依赖）
        event_class_name = type(event).__name__

        # 基础事件信息
        base_json = {
            "timestamp": time.time(),
        }

        try:
            if "TokenUsageEvent" in event_class_name:
                usage = getattr(event, "usage", None)
                base_json.update(
                    {
                        "event_type": "token_usage",
                        "data": {
                            "usage": (
                                self._serialize_usage_object(usage) if usage else None
                            )
                        },
                    }
                )
            elif "WindowLengthChangeEvent" in event_class_name:
                tokens_used = getattr(event, "tokens_used", 0)
                base_json.update(
                    {
                        "event_type": "window_change",
                        "data": {"tokens_used": tokens_used},
                    }
                )
            elif "LLMThinkingEvent" in event_class_name:
                text = getattr(event, "text", "")
                base_json.update({"event_type": "llm_thinking", "data": {"text": text}})
            elif "LLMOutputEvent" in event_class_name:
                text = getattr(event, "text", "")
                base_json.update({"event_type": "llm_output", "data": {"text": text}})
            elif "ToolCallEvent" in event_class_name:
                tool = getattr(event, "tool", None)
                tool_name = type(tool).__name__ if tool else "Unknown Tool"
                tool_args = self._serialize_tool_object(tool) if tool else {}
                base_json.update(
                    {
                        "event_type": "tool_call",
                        "data": {"tool_name": tool_name, "args": tool_args},
                    }
                )
            elif "ToolResultEvent" in event_class_name:
                tool_name = getattr(event, "tool_name", "Unknown")
                result = getattr(event, "result", None)
                result_data = self._serialize_tool_result(result) if result else {}
                base_json.update(
                    {
                        "event_type": "tool_result",
                        "data": {"tool_name": tool_name, **result_data},
                    }
                )
            elif "CompletionEvent" in event_class_name:
                completion = getattr(event, "completion", None)
                completion_data = (
                    self._serialize_completion_object(completion) if completion else {}
                )
                base_json.update({"event_type": "completion", "data": completion_data})
            elif "PlanModeRespondEvent" in event_class_name:
                completion = getattr(event, "completion", None)
                response = (
                    getattr(completion, "response", "Plan completed")
                    if completion
                    else "Plan completed"
                )
                base_json.update(
                    {"event_type": "plan_mode_respond", "data": {"result": response}}
                )
            elif "ErrorEvent" in event_class_name:
                message = getattr(event, "message", "Unknown error")
                base_json.update(
                    {
                        "event_type": "error",
                        "data": {"error": message, "error_type": "Error"},
                    }
                )
            elif "ConversationIdEvent" in event_class_name:
                conversation_id = getattr(event, "conversation_id", "")
                base_json.update(
                    {
                        "event_type": "conversation_id",
                        "data": {"conversation_id": conversation_id},
                    }
                )
            elif "PreCommitEvent" in event_class_name:
                # event.commit_result 和 event.tpe 一定存在
                commit_result = event.commit_result
                commit_data = self._serialize_commit_result(commit_result)
                base_json.update(
                    {
                        "event_type": "pre_commit",
                        "data": {"commit_result": commit_data, "tpe": event.tpe},
                    }
                )
            elif "CommitEvent" in event_class_name:
                # event.commit_result 和 event.tpe 一定存在
                commit_result = event.commit_result
                commit_data = self._serialize_commit_result(commit_result)
                base_json.update(
                    {
                        "event_type": "commit",
                        "data": {"commit_result": commit_data, "tpe": event.tpe},
                    }
                )
            elif "PullRequestEvent" in event_class_name:
                # event.pull_request_result 一定存在
                pr_result = event.pull_request_result
                pr_data = self._serialize_pr_result(pr_result)
                base_json.update(
                    {
                        "event_type": "pull_request",
                        "data": {"pull_request_result": pr_data},
                    }
                )
            else:
                # 处理旧格式的事件（有 event_type 和 data 属性）
                if hasattr(event, "event_type") and hasattr(event, "data"):
                    base_json.update(
                        {
                            "event_type": event.event_type,
                            "data": (
                                event.data
                                if isinstance(event.data, dict)
                                else {"content": str(event.data)}
                            ),
                        }
                    )
                else:
                    # 未知事件类型，尝试序列化所有属性
                    base_json.update(
                        {
                            "event_type": "unknown",
                            "data": self._serialize_object_attributes(event),
                        }
                    )
        except Exception as e:
            # 如果序列化失败，返回基本信息
            base_json.update(
                {
                    "event_type": "serialization_error",
                    "data": {"error": str(e), "original_class": event_class_name},
                }
            )

        return base_json

    def _serialize_usage_object(self, usage) -> dict:
        """序列化使用情况对象"""
        try:
            return {
                "input_tokens_count": getattr(usage, "input_tokens_count", 0),
                "generated_tokens_count": getattr(usage, "generated_tokens_count", 0),
                "total_tokens": getattr(usage, "input_tokens_count", 0)
                + getattr(usage, "generated_tokens_count", 0),
            }
        except:
            return {"error": "Failed to serialize usage object"}

    def _serialize_tool_object(self, tool) -> dict:
        """序列化工具对象"""
        try:
            if hasattr(tool, "__dict__"):
                result = {}
                for key, value in tool.__dict__.items():
                    if not key.startswith("_"):
                        try:
                            # 尝试序列化值，如果失败则转为字符串
                            json.dumps(value)
                            result[key] = value
                        except:
                            result[key] = str(value)
                return result
            else:
                return {"tool": str(tool)}
        except:
            return {"error": "Failed to serialize tool object"}

    def _serialize_tool_result(self, result) -> dict:
        """序列化工具结果对象"""
        try:
            return {
                "success": getattr(result, "success", True),
                "message": getattr(result, "message", ""),
                "content": self._serialize_content(getattr(result, "content", None)),
            }
        except:
            return {"error": "Failed to serialize tool result"}

    def _serialize_completion_object(self, completion) -> dict:
        """序列化完成对象"""
        try:
            return {
                "result": getattr(completion, "result", "Task completed successfully"),
                "command": getattr(completion, "command", None),
            }
        except:
            return {"error": "Failed to serialize completion object"}

    def _serialize_content(self, content) -> Any:
        """序列化内容，确保可以 JSON 序列化"""
        try:
            if content is None:
                return None
            # 尝试 JSON 序列化测试
            json.dumps(content)
            return content
        except:
            # 如果不能序列化，转为字符串
            return str(content)

    def _serialize_object_attributes(self, obj) -> dict:
        """序列化对象的所有属性"""
        try:
            result = {}
            if hasattr(obj, "__dict__"):
                for key, value in obj.__dict__.items():
                    if not key.startswith("_"):
                        result[key] = self._serialize_content(value)
            return result
        except:
            return {"error": "Failed to serialize object attributes"}

    def _serialize_commit_result(self, commit_result) -> dict:
        """序列化 Git 提交结果对象"""
        try:
            return {
                "success": (
                    commit_result.success
                    if hasattr(commit_result, "success")
                    else False
                ),
                "message": (
                    commit_result.message if hasattr(commit_result, "message") else ""
                ),
                "commit_hash": (
                    commit_result.commit_hash
                    if hasattr(commit_result, "commit_hash")
                    else ""
                ),
                "files_changed": (
                    commit_result.files_changed
                    if hasattr(commit_result, "files_changed")
                    else []
                ),
                "error": (
                    commit_result.error if hasattr(commit_result, "error") else None
                ),
            }
        except:
            return {"error": "Failed to serialize commit result"}

    def _serialize_pr_result(self, pr_result) -> dict:
        """序列化 Pull Request 结果对象"""
        try:
            return {
                "success": (
                    pr_result.success if hasattr(pr_result, "success") else False
                ),
                "pr_number": (
                    pr_result.pr_number if hasattr(pr_result, "pr_number") else None
                ),
                "pr_url": pr_result.pr_url if hasattr(pr_result, "pr_url") else "",
                "pr_id": pr_result.pr_id if hasattr(pr_result, "pr_id") else None,
                "error_message": (
                    pr_result.error_message
                    if hasattr(pr_result, "error_message")
                    else None
                ),
                "error_code": (
                    pr_result.error_code if hasattr(pr_result, "error_code") else None
                ),
                "platform": (
                    pr_result.platform if hasattr(pr_result, "platform") else None
                ),
                "raw_response": self._serialize_content(
                    pr_result.raw_response
                    if hasattr(pr_result, "raw_response")
                    else None
                ),
                "retry_after": (
                    pr_result.retry_after if hasattr(pr_result, "retry_after") else None
                ),
            }
        except:
            return {"error": "Failed to serialize pull request result"}

    def _format_tool_display(self, tool_name: str, tool_args: Dict[str, Any]) -> str:
        """
        格式化工具调用显示内容

        Args:
            tool_name: 工具名称
            tool_args: 工具参数

        Returns:
            str: 格式化后的显示内容
        """
        if not tool_args:
            return f"[bold]Tool:[/bold] {tool_name}"

        content_parts = [f"[bold]Tool:[/bold] {tool_name}"]

        for key, value in tool_args.items():
            if isinstance(value, str) and len(value) > 100:
                value = f"{value[:50]}...{value[-50:]}"
            content_parts.append(f"[bold]{key}:[/bold] {value}")

        return "\n".join(content_parts)

    def _process_token_usage_event(self, usage):
        """
        处理 TokenUsageEvent，累计 token 使用情况

        Args:
            usage: SingleOutputMeta 对象
        """
        try:
            # 正确提取 SingleOutputMeta 对象的属性
            input_tokens = getattr(usage, "input_tokens_count", 0)
            output_tokens = getattr(usage, "generated_tokens_count", 0)

            # 获取模型信息用于定价
            try:
                from autocoder.utils import llms as llm_utils

                # 这里需要获取 LLM 实例，但在 SDK 中可能不直接可用
                # 暂时使用默认模型名称
                model_name = self.options.model or "unknown"
                model_info = llm_utils.get_model_info(model_name, "lite") or {}
                input_price = model_info.get("input_price", 0.0)
                output_price = model_info.get("output_price", 0.0)
            except:
                # 如果获取模型信息失败，使用默认值
                model_name = self.options.model or "unknown"
                input_price = 0.0
                output_price = 0.0

            # 计算成本
            input_cost = (input_tokens * input_price) / 1000000  # 转换为百万token单位
            output_cost = (output_tokens * output_price) / 1000000

            # 累计token使用情况
            self._accumulated_token_usage["model_name"] = model_name
            self._accumulated_token_usage["input_tokens"] += input_tokens
            self._accumulated_token_usage["output_tokens"] += output_tokens
            self._accumulated_token_usage["input_cost"] += input_cost
            self._accumulated_token_usage["output_cost"] += output_cost

            # 显示当前的 token 使用情况
            total_tokens = input_tokens + output_tokens
            self._console.print(
                f"[dim]Token usage: Input={input_tokens}, Output={output_tokens}, Total={total_tokens}[/dim]"
            )

        except Exception as e:
            self._console.print(
                f"[dim red]Error processing token usage: {str(e)}[/dim red]"
            )

    def _print_final_token_usage(self):
        """
        打印最终的累计 token 使用情况
        """
        try:
            if self._accumulated_token_usage["input_tokens"] > 0:
                from autocoder.common.printer import Printer

                printer = Printer()
                printer.print_in_terminal(
                    "code_generation_complete",
                    duration=0.0,
                    input_tokens=self._accumulated_token_usage["input_tokens"],
                    output_tokens=self._accumulated_token_usage["output_tokens"],
                    input_cost=self._accumulated_token_usage["input_cost"],
                    output_cost=self._accumulated_token_usage["output_cost"],
                    speed=0.0,
                    model_names=self._accumulated_token_usage["model_name"],
                    sampling_count=1,
                )
        except Exception as e:
            # 如果打印失败，使用简单的格式
            total_tokens = (
                self._accumulated_token_usage["input_tokens"]
                + self._accumulated_token_usage["output_tokens"]
            )
            total_cost = (
                self._accumulated_token_usage["input_cost"]
                + self._accumulated_token_usage["output_cost"]
            )
            self._console.print(
                Panel(
                    f"总计 Token 使用: {total_tokens} (输入: {self._accumulated_token_usage['input_tokens']}, 输出: {self._accumulated_token_usage['output_tokens']})\n"
                    f"总计成本: ${total_cost:.6f}",
                    title="📊 Token 使用统计",
                    border_style="cyan",
                )
            )

    def _reset_token_usage(self):
        """
        重置累计的 token 使用情况
        """
        self._accumulated_token_usage = {
            "model_name": "",
            "input_tokens": 0,
            "output_tokens": 0,
            "input_cost": 0.0,
            "output_cost": 0.0,
        }

    def _format_tool_result_content(
        self, content: Any, tool_name: str = ""
    ) -> str | Syntax:
        """
        格式化工具结果内容

        Args:
            content: 结果内容
            tool_name: 工具名称（用于推断语法类型）

        Returns:
            str | Syntax: 格式化后的内容或语法高亮对象
        """

        def _truncate_content(content_str: str) -> str:
            if len(content_str) > 500:
                return f"{content_str[:200]}\n...\n{content_str[-200:]}"
            return content_str

        try:
            # Remove hints from content before processing
            processed_content = content
            if isinstance(content, str) and has_hint_in_text(content):
                processed_content = extract_content_from_text(content)

            if isinstance(processed_content, (dict, list)):
                import json

                content_str = json.dumps(
                    processed_content, indent=2, ensure_ascii=False
                )
                return Syntax(
                    _truncate_content(content_str),
                    "json",
                    theme="default",
                    line_numbers=False,
                )

            elif isinstance(processed_content, str):
                # 检查是否是多行内容或代码
                if (
                    "\n" in processed_content
                    or processed_content.strip().startswith("<")
                    or len(processed_content) > 200
                ):
                    # 推断语法类型
                    lexer = "text"
                    if "ReadFile" in tool_name:
                        if any(ext in processed_content for ext in [".py", "python"]):
                            lexer = "python"
                        elif any(
                            ext in processed_content for ext in [".js", "javascript"]
                        ):
                            lexer = "javascript"
                        elif any(
                            ext in processed_content for ext in [".ts", "typescript"]
                        ):
                            lexer = "typescript"
                        elif any(
                            ext in processed_content
                            for ext in [".html", "<!DOCTYPE", "<html"]
                        ):
                            lexer = "html"
                        elif any(ext in processed_content for ext in [".css", "{"]):
                            lexer = "css"
                        elif any(ext in processed_content for ext in [".json", "{"]):
                            lexer = "json"
                        elif any(ext in processed_content for ext in [".xml", "<?xml"]):
                            lexer = "xml"
                        elif any(ext in processed_content for ext in [".md", "#"]):
                            lexer = "markdown"
                    elif "ExecuteCommand" in tool_name or "Shell" in tool_name:
                        lexer = "shell"
                    elif processed_content.strip().startswith(
                        "{"
                    ) or processed_content.strip().startswith("["):
                        lexer = "json"

                    return Syntax(
                        _truncate_content(processed_content),
                        lexer,
                        theme="default",
                        line_numbers=True,
                    )
                else:
                    return _truncate_content(processed_content)
            else:
                return _truncate_content(str(processed_content))

        except Exception:
            return _truncate_content(str(content))

    async def query_stream(
        self, prompt: str, show_terminal: Optional[bool] = None
    ) -> AsyncIterator[StreamEvent]:
        """
        异步流式查询 - 使用 run_auto_command

        Args:
            prompt: 查询提示
            show_terminal: 是否显示到终端，None时使用配置中的verbose设置

        Yields:
            StreamEvent: 响应事件流

        Raises:
            BridgeError: 桥接层错误
        """
        # 如果没有明确指定show_terminal，使用verbose配置
        if show_terminal is None:
            show_terminal = self.options.verbose

        # 如果verbose为False，则强制show_terminal为False
        if not self.options.verbose:
            show_terminal = False

        try:
            # 重置累计的 token 使用情况
            self._reset_token_usage()

            # 先返回用户消息
            user_message = StreamEvent(event_type="start", data={"query": prompt})
            yield user_message

            # 在线程池中执行同步调用
            loop = asyncio.get_event_loop()

            # 使用 run_auto_command 进行代码修改，传递 cancel_token
            event_stream = await loop.run_in_executor(
                self._executor,
                self._sync_run_auto_command,
                prompt,
                False,  # pre_commit
                None,  # pr (使用默认值)
                None,  # extra_args
            )

            # 处理事件流并转换为消息
            for event in event_stream:
                # 渲染事件到终端
                self._render_stream_event(event, show_terminal)
                yield event

            # 打印最终的累计 token 使用情况
            if show_terminal:
                self._print_final_token_usage()

        except Exception as e:
            # 在异常时也打印累计的 token 使用情况
            if show_terminal:
                self._print_final_token_usage()
            raise e

    def query_sync(
        self, prompt: str, show_terminal: Optional[bool] = None
    ) -> List[StreamEvent]:
        """
        同步查询 - 通过调用 query_stream 实现

        Args:
            prompt: 查询提示
            show_terminal: 是否显示到终端，None时使用配置中的verbose设置

        Returns:
            List[StreamEvent]: 事件列表，包含用户查询和响应事件

        Raises:
            BridgeError: 桥接层错误
        """
        try:
            # 收集所有事件
            events = []

            # 运行异步查询流并收集结果
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)

            try:

                async def collect_events():
                    async for event in self.query_stream(prompt, show_terminal):
                        events.append(event)

                loop.run_until_complete(collect_events())
            finally:
                loop.close()

            return events

        except Exception as e:
            raise e

    def _sync_run_auto_command(
        self,
        prompt: str,
        pre_commit: bool = False,
        pr: Optional[bool] = None,
        extra_args: Optional[Dict[str, Any]] = None,
    ) -> Iterator[StreamEvent]:
        """
        内部同步调用 run_auto_command

        Args:
            prompt: 查询提示
            pre_commit: 是否预提交
            pr: 是否创建 PR，如果为None则使用配置中的值
            extra_args: 额外参数

        Returns:
            Iterator[StreamEvent]: 事件流
        """
        # 如果没有明确指定pr参数，使用配置中的值
        if pr is None:
            pr = self.options.pr

        effective_extra_args = self._build_extra_args(extra_args)

        return self.bridge.call_run_auto_command(
            query=prompt,
            pre_commit=pre_commit,
            pr=pr,
            cancel_token=self.options.cancel_token,
            extra_args=effective_extra_args,
            stream=True,
        )

    def get_project_memory(self) -> Dict[str, Any]:
        """
        获取项目内存状态

        Returns:
            Dict[str, Any]: 项目内存数据
        """
        return self.bridge.get_memory()

    def save_project_memory(self, memory_data: Dict[str, Any]) -> None:
        """
        保存项目内存状态

        Args:
            memory_data: 内存数据
        """
        self.bridge.save_memory(memory_data)

    def get_project_config(self) -> Dict[str, Any]:
        """
        获取项目配置

        Returns:
            Dict[str, Any]: 项目配置
        """
        return self.bridge.get_project_config()

    def run_workflow(
        self,
        workflow: str,
        prompt: Optional[str] = None,
        vars_override: Optional[Dict[str, Any]] = None,
    ) -> Any:
        """
        运行指定的 workflow

        Args:
            workflow: workflow 名称或文件路径
            prompt: 可选的提示内容，会注入到 workflow 的 vars.query 中
            vars_override: 可选的变量覆盖字典

        Returns:
            WorkflowResult: workflow 执行结果

        Raises:
            BridgeError: 桥接层错误
        """
        return self.bridge.run_workflow(
            workflow=workflow,
            prompt=prompt,
            model=self.options.model,
            vars_override=vars_override,
        )

    def __del__(self):
        """清理资源"""
        if hasattr(self, "_executor"):
            self._executor.shutdown(wait=False)



import pytest
import json
from datetime import datetime
from pathlib import Path

from autocoder.common.file_checkpoint.models import (
    FileChange, ChangeRecord, DiffResult, ApplyResult, UndoResult
)

class TestFileChange:
    """FileChange类的单元测试"""
    
    def test_init(self):
        """测试FileChange初始化"""
        fc = FileChange(
            file_path="test.py",
            content="print('hello')",
            is_new=True,
            is_deletion=False
        )
        
        assert fc.file_path == "test.py"
        assert fc.content == "print('hello')"
        assert fc.is_new is True
        assert fc.is_deletion is False
    
    def test_from_dict(self):
        """测试从字典创建FileChange对象"""
        data = {
            'file_path': 'test.py',
            'content': "print('hello')",
            'is_new': True,
            'is_deletion': False
        }
        
        fc = FileChange.from_dict(data)
        
        assert fc.file_path == "test.py"
        assert fc.content == "print('hello')"
        assert fc.is_new is True
        assert fc.is_deletion is False
    
    def test_to_dict(self):
        """测试将FileChange对象转换为字典"""
        fc = FileChange(
            file_path="test.py",
            content="print('hello')",
            is_new=True,
            is_deletion=False
        )
        
        data = fc.to_dict()
        
        assert data['file_path'] == "test.py"
        assert data['content'] == "print('hello')"
        assert data['is_new'] is True
        assert data['is_deletion'] is False
    
    def test_default_values(self):
        """测试默认值"""
        fc = FileChange(
            file_path="test.py",
            content="print('hello')"
        )
        
        assert fc.is_new is False
        assert fc.is_deletion is False
    
    def test_from_dict_missing_optionals(self):
        """测试从缺少可选字段的字典创建"""
        data = {
            'file_path': 'test.py',
            'content': "print('hello')"
        }
        
        fc = FileChange.from_dict(data)
        
        assert fc.file_path == "test.py"
        assert fc.content == "print('hello')"
        assert fc.is_new is False
        assert fc.is_deletion is False


class TestChangeRecord:
    """ChangeRecord类的单元测试"""
    
    def test_init(self):
        """测试ChangeRecord初始化"""
        cr = ChangeRecord(
            change_id="123",
            timestamp=1234567890.0,
            file_path="test.py",
            backup_id="456",
            is_new=True,
            is_deletion=False,
            group_id="789"
        )
        
        assert cr.change_id == "123"
        assert cr.timestamp == 1234567890.0
        assert cr.file_path == "test.py"
        assert cr.backup_id == "456"
        assert cr.is_new is True
        assert cr.is_deletion is False
        assert cr.group_id == "789"
    
    def test_create(self):
        """测试创建一个新的变更记录"""
        before_time = datetime.now().timestamp()
        
        cr = ChangeRecord.create(
            file_path="test.py",
            backup_id="456",
            is_new=True,
            is_deletion=False,
            group_id="789"
        )
        
        after_time = datetime.now().timestamp()
        
        assert cr.file_path == "test.py"
        assert cr.backup_id == "456"
        assert cr.is_new is True
        assert cr.is_deletion is False
        assert cr.group_id == "789"
        
        # UUID和时间戳应该是自动生成的
        assert cr.change_id is not None
        assert len(cr.change_id) > 0
        assert before_time <= cr.timestamp <= after_time
    
    def test_from_dict(self):
        """测试从字典创建ChangeRecord对象"""
        data = {
            'change_id': '123',
            'timestamp': 1234567890.0,
            'file_path': 'test.py',
            'backup_id': '456',
            'is_new': True,
            'is_deletion': False,
            'group_id': '789'
        }
        
        cr = ChangeRecord.from_dict(data)
        
        assert cr.change_id == "123"
        assert cr.timestamp == 1234567890.0
        assert cr.file_path == "test.py"
        assert cr.backup_id == "456"
        assert cr.is_new is True
        assert cr.is_deletion is False
        assert cr.group_id == "789"
    
    def test_to_dict(self):
        """测试将ChangeRecord对象转换为字典"""
        cr = ChangeRecord(
            change_id="123",
            timestamp=1234567890.0,
            file_path="test.py",
            backup_id="456",
            is_new=True,
            is_deletion=False,
            group_id="789"
        )
        
        data = cr.to_dict()
        
        assert data['change_id'] == "123"
        assert data['timestamp'] == 1234567890.0
        assert data['file_path'] == "test.py"
        assert data['backup_id'] == "456"
        assert data['is_new'] is True
        assert data['is_deletion'] is False
        assert data['group_id'] == "789"
    
    def test_default_values(self):
        """测试默认值"""
        cr = ChangeRecord(
            change_id="123",
            timestamp=1234567890.0,
            file_path="test.py",
            backup_id="456"
        )
        
        assert cr.is_new is False
        assert cr.is_deletion is False
        assert cr.group_id is None
    
    def test_from_dict_missing_optionals(self):
        """测试从缺少可选字段的字典创建"""
        data = {
            'change_id': '123',
            'timestamp': 1234567890.0,
            'file_path': 'test.py',
            'backup_id': '456'
        }
        
        cr = ChangeRecord.from_dict(data)
        
        assert cr.change_id == "123"
        assert cr.timestamp == 1234567890.0
        assert cr.file_path == "test.py"
        assert cr.backup_id == "456"
        assert cr.is_new is False
        assert cr.is_deletion is False
        assert cr.group_id is None


class TestDiffResult:
    """DiffResult类的单元测试"""
    
    def test_init(self):
        """测试DiffResult初始化"""
        dr = DiffResult(
            file_path="test.py",
            old_content="print('hello')",
            new_content="print('world')",
            is_new=False,
            is_deletion=False
        )
        
        assert dr.file_path == "test.py"
        assert dr.old_content == "print('hello')"
        assert dr.new_content == "print('world')"
        assert dr.is_new is False
        assert dr.is_deletion is False
    
    def test_default_values(self):
        """测试默认值"""
        dr = DiffResult(
            file_path="test.py",
            old_content="print('hello')",
            new_content="print('world')"
        )
        
        assert dr.is_new is False
        assert dr.is_deletion is False
    
    def test_get_diff_summary_modify(self):
        """测试获取修改文件的差异摘要"""
        dr = DiffResult(
            file_path="test.py",
            old_content="line1\nline2\nline3",
            new_content="line1\nline2\nline3\nline4"
        )
        
        summary = dr.get_diff_summary()
        
        assert "修改文件" in summary
        assert "test.py" in summary
        assert "原始行数: 3" in summary
        assert "新行数: 4" in summary
    
    def test_get_diff_summary_new(self):
        """测试获取新文件的差异摘要"""
        dr = DiffResult(
            file_path="test.py",
            old_content=None,
            new_content="print('hello')",
            is_new=True
        )
        
        summary = dr.get_diff_summary()
        
        assert "新文件" in summary
        assert "test.py" in summary
    
    def test_get_diff_summary_delete(self):
        """测试获取删除文件的差异摘要"""
        dr = DiffResult(
            file_path="test.py",
            old_content="print('hello')",
            new_content="",
            is_deletion=True
        )
        
        summary = dr.get_diff_summary()
        
        assert "删除文件" in summary
        assert "test.py" in summary


class TestApplyResult:
    """ApplyResult类的单元测试"""
    
    def test_init(self):
        """测试ApplyResult初始化"""
        ar = ApplyResult(success=True)
        
        assert ar.success is True
        assert ar.change_ids == []
        assert ar.errors == {}
        assert ar.has_errors is False
    
    def test_add_error(self):
        """测试添加错误信息"""
        ar = ApplyResult(success=True)
        
        assert ar.success is True
        assert ar.has_errors is False
        
        ar.add_error("test.py", "文件不存在")
        
        assert ar.success is False
        assert ar.has_errors is True
        assert "test.py" in ar.errors
        assert ar.errors["test.py"] == "文件不存在"
    
    def test_add_change_id(self):
        """测试添加成功应用的变更ID"""
        ar = ApplyResult(success=True)
        
        assert len(ar.change_ids) == 0
        
        ar.add_change_id("123")
        ar.add_change_id("456")
        
        assert len(ar.change_ids) == 2
        assert "123" in ar.change_ids
        assert "456" in ar.change_ids


class TestUndoResult:
    """UndoResult类的单元测试"""
    
    def test_init(self):
        """测试UndoResult初始化"""
        ur = UndoResult(success=True)
        
        assert ur.success is True
        assert ur.restored_files == []
        assert ur.errors == {}
        assert ur.has_errors is False
    
    def test_add_error(self):
        """测试添加错误信息"""
        ur = UndoResult(success=True)
        
        assert ur.success is True
        assert ur.has_errors is False
        
        ur.add_error("test.py", "文件不存在")
        
        assert ur.success is False
        assert ur.has_errors is True
        assert "test.py" in ur.errors
        assert ur.errors["test.py"] == "文件不存在"
    
    def test_add_restored_file(self):
        """测试添加成功恢复的文件"""
        ur = UndoResult(success=True)
        
        assert len(ur.restored_files) == 0
        
        ur.add_restored_file("test.py")
        ur.add_restored_file("main.py")
        
        assert len(ur.restored_files) == 2
        assert "test.py" in ur.restored_files
        assert "main.py" in ur.restored_files 


import os
from pathlib import Path
import json
import pytest

from autocoder.chat.conf_command import handle_conf_command
from autocoder.common.core_config import (
    get_memory_manager,
    get_global_memory_manager,
)
import autocoder.common.core_config.main_manager as main_manager_mod


def test_project_shorthand_sets_value(tmp_path: Path, monkeypatch: pytest.MonkeyPatch):
    # Use an isolated project root
    project_root = tmp_path / "proj"
    project_root.mkdir(parents=True, exist_ok=True)
    monkeypatch.chdir(project_root)

    # Set via shorthand
    msg = handle_conf_command("project_key:123")
    assert "project_key" in msg

    # Verify via manager (project scope)
    mm = get_memory_manager(str(project_root))
    assert mm.get_config("project_key", source="project") == 123


def test_global_shorthand_sets_value(tmp_path: Path, monkeypatch: pytest.MonkeyPatch):
    # Prepare isolated fake HOME: ~/.auto-coder/plugins/chat-auto-coder
    fake_home = tmp_path / "home"
    global_conf_dir = fake_home / ".auto-coder" / "plugins" / "chat-auto-coder"
    global_conf_dir.mkdir(parents=True, exist_ok=True)

    # Monkeypatch the global config dir resolver used by get_global_memory_manager
    monkeypatch.setattr(
        main_manager_mod,
        "get_global_config_dir",
        lambda: global_conf_dir,
        raising=True,
    )

    # Use global shorthand to set value
    msg = handle_conf_command("/global metaso_api_key:xxxx")
    assert "metaso_api_key" in msg
    assert "(global)" in msg  # scope suffix present

    # Verify value persisted in the global manager's project scope
    gmm = get_global_memory_manager()
    assert gmm.get_config("metaso_api_key", source="project") == "xxxx"



"""
GitLab API 提供者实现
"""
from typing import List
from .github_provider import GitHubProvider
from ..models import RepoInfo, PRData, PRResult, PRInfo


class GitLabProvider(GitHubProvider):
    """GitLab API 提供者（基于GitHub提供者）"""
    
    def _get_auth_header(self) -> str:
        """获取认证头"""
        return f"Bearer {self.config.token}"
    
    def create_pr(self, repo_info: RepoInfo, pr_data: PRData) -> PRResult:
        """创建 Merge Request (GitLab的PR称为MR)"""
        # 简化实现，实际应该调用GitLab API
        return super().create_pr(repo_info, pr_data)
    
    def list_prs(
        self, 
        repo_info: RepoInfo, 
        state: str = "opened",  # GitLab使用"opened"而不是"open"
        per_page: int = 30,
        page: int = 1
    ) -> List[PRInfo]:
        """列出仓库的MR"""
        return super().list_prs(repo_info, state, per_page, page)


import pydantic
from typing import List
from autocoder.common import SourceCode

class TokenLimiterResult(pydantic.BaseModel):
    # 注意， docs 只保留结果文档
    docs: List[SourceCode]
    # 注意， raw_docs 保留所有文档
    raw_docs: List[SourceCode]
    input_tokens_counts: List[int]
    generated_tokens_counts: List[int]
    durations: List[float]
    model_name: str = "unknown"


"""
Backup and restore functionality for conversation management.

This package provides backup and restore capabilities for conversation data,
including incremental and full backups, version management, and data recovery.
"""

from .backup_manager import BackupManager
from .restore_manager import RestoreManager

__all__ = [
    'BackupManager',
    'RestoreManager',
] 


"""
文件变更管理模块的数据模型

定义了表示文件变更、变更记录和操作结果的数据类。
"""

import os
import uuid
from dataclasses import dataclass, field
from datetime import datetime
from typing import Dict, List, Optional, Union


@dataclass
class FileChange:
    """表示单个文件的变更信息"""
    file_path: str
    content: str
    is_new: bool = False
    is_deletion: bool = False
    
    @classmethod
    def from_dict(cls, data: Dict) -> 'FileChange':
        """从字典创建FileChange对象"""
        return cls(
            file_path=data['file_path'],
            content=data['content'],
            is_new=data.get('is_new', False),
            is_deletion=data.get('is_deletion', False)
        )
    
    def to_dict(self) -> Dict:
        """将FileChange对象转换为字典"""
        return {
            'file_path': self.file_path,
            'content': self.content,
            'is_new': self.is_new,
            'is_deletion': self.is_deletion
        }


@dataclass
class ChangeRecord:
    """表示一条变更记录，包含变更的元数据和详细信息"""
    change_id: str
    timestamp: float
    file_path: str
    backup_id: Optional[str]
    is_new: bool = False
    is_deletion: bool = False
    group_id: Optional[str] = None
    
    @classmethod
    def create(cls, file_path: str, backup_id: Optional[str], 
               is_new: bool = False, is_deletion: bool = False, 
               group_id: Optional[str] = None) -> 'ChangeRecord':
        """创建一个新的变更记录"""
        return cls(
            change_id=str(uuid.uuid4()),
            timestamp=datetime.now().timestamp(),
            file_path=file_path,
            backup_id=backup_id,
            is_new=is_new,
            is_deletion=is_deletion,
            group_id=group_id
        )
    
    @classmethod
    def from_dict(cls, data: Dict) -> 'ChangeRecord':
        """从字典创建ChangeRecord对象"""
        return cls(
            change_id=data['change_id'],
            timestamp=data['timestamp'],
            file_path=data['file_path'],
            backup_id=data.get('backup_id'),
            is_new=data.get('is_new', False),
            is_deletion=data.get('is_deletion', False),
            group_id=data.get('group_id')
        )
    
    def to_dict(self) -> Dict:
        """将ChangeRecord对象转换为字典"""
        return {
            'change_id': self.change_id,
            'timestamp': self.timestamp,
            'file_path': self.file_path,
            'backup_id': self.backup_id,
            'is_new': self.is_new,
            'is_deletion': self.is_deletion,
            'group_id': self.group_id
        }


@dataclass
class DiffResult:
    """表示文件差异比较的结果"""
    file_path: str
    old_content: Optional[str]
    new_content: str
    is_new: bool = False
    is_deletion: bool = False
    
    def get_diff_summary(self) -> str:
        """获取差异摘要"""
        if self.is_new:
            return f"新文件: {self.file_path}"
        elif self.is_deletion:
            return f"删除文件: {self.file_path}"
        else:
            old_lines = 0 if self.old_content is None else len(self.old_content.splitlines())
            new_lines = len(self.new_content.splitlines())
            return f"修改文件: {self.file_path} (原始行数: {old_lines}, 新行数: {new_lines})"


@dataclass
class ApplyResult:
    """表示变更应用的结果"""
    success: bool
    change_ids: List[str] = field(default_factory=list)
    errors: Dict[str, str] = field(default_factory=dict)
    
    @property
    def has_errors(self) -> bool:
        """是否有错误"""
        return len(self.errors) > 0
    
    def add_error(self, file_path: str, error_message: str) -> None:
        """添加错误信息"""
        self.errors[file_path] = error_message
        self.success = False
    
    def add_change_id(self, change_id: str) -> None:
        """添加成功应用的变更ID"""
        self.change_ids.append(change_id)


@dataclass
class UndoResult:
    """表示变更撤销的结果"""
    success: bool
    restored_files: List[str] = field(default_factory=list)
    errors: Dict[str, str] = field(default_factory=dict)
    
    @property
    def has_errors(self) -> bool:
        """是否有错误"""
        return len(self.errors) > 0
    
    def add_error(self, file_path: str, error_message: str) -> None:
        """添加错误信息"""
        self.errors[file_path] = error_message
        self.success = False
    
    def add_restored_file(self, file_path: str) -> None:
        """添加成功恢复的文件"""
        self.restored_files.append(file_path)




"""
Metaso 集成使用示例

该文件展示如何使用集成了 Metaso 的 web_search_tool 和 web_crawl_tool。
"""

import os
import sys
from typing import Dict, Any

# 添加项目路径
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '../../..'))

from autocoder.rag.tools.metaso_sdk import MetasoClient
from autocoder.rag.tools.web_search_tool import WebSearchTool, WebSearchToolResolver
from autocoder.rag.tools.web_crawl_tool import WebCrawlTool, WebCrawlToolResolver
from unittest.mock import Mock


def example_direct_metaso_sdk():
    """直接使用 Metaso SDK 的示例"""
    print("=" * 60)
    print("示例 1: 直接使用 Metaso SDK")
    print("=" * 60)
    
    # 从环境变量获取 API key
    api_key = os.getenv('METASO_API_KEY')
    if not api_key:
        print("\n⚠️  请先设置环境变量 METASO_API_KEY")
        print("   export METASO_API_KEY='your-api-key-here'")
        return
    
    # 创建客户端
    client = MetasoClient(api_key=api_key)
    
    # 1. 搜索示例
    print("\n1. 搜索示例:")
    search_result = client.search(
        query="Python 编程最佳实践",
        size=3,
        include_summary=False
    )
    
    if search_result.success:
        print(f"   找到 {len(search_result.webpages)} 个结果:")
        for idx, page in enumerate(search_result.webpages, 1):
            print(f"\n   {idx}. {page.title}")
            print(f"      URL: {page.link}")
            print(f"      摘要: {page.snippet[:100]}...")
            if page.date:
                print(f"      日期: {page.date}")
    else:
        print(f"   搜索失败: {search_result.error}")
    
    # 2. 读取网页示例
    print("\n2. 读取网页示例:")
    if search_result.success and search_result.webpages:
        first_url = search_result.webpages[0].link
        print(f"   读取 URL: {first_url}")
        
        content = client.read(first_url)
        if not content.startswith("Error:"):
            print(f"   成功读取，内容长度: {len(content)} 字符")
            print(f"   内容预览:\n{content[:500]}...")
        else:
            print(f"   读取失败: {content}")


def example_web_search_tool():
    """使用 WebSearchTool 的示例"""
    print("\n" + "=" * 60)
    print("示例 2: 使用 WebSearchTool (集成 Metaso)")
    print("=" * 60)
    
    # 创建 mock agent 和 args
    mock_agent = Mock()
    mock_args = Mock()
    
    # 设置 API keys (可以同时设置，系统会自动选择)
    mock_args.metaso_api_key = os.getenv('METASO_API_KEY')
    mock_args.firecrawl_api_key = os.getenv('FIRECRAWL_API_KEY')
    
    if not mock_args.metaso_api_key:
        print("\n⚠️  请先设置环境变量 METASO_API_KEY")
        return
    
    # 创建搜索工具 - 明确指定使用 Metaso
    tool = WebSearchTool(
        query="人工智能在医疗领域的应用",
        limit=5,
        provider="metaso",  # 明确指定使用 Metaso
        scrape_options='{"include_summary": true}'
    )
    
    # 创建解析器并执行
    resolver = WebSearchToolResolver(mock_agent, tool, mock_args)
    result = resolver.resolve()
    
    if result.success:
        print(f"\n搜索成功: {result.message}")
        print(f"找到 {len(result.content)} 个结果:")
        
        for idx, item in enumerate(result.content, 1):
            print(f"\n{idx}. {item['title']}")
            print(f"   URL: {item['url']}")
            print(f"   描述: {item['description'][:100]}...")
            print(f"   来源: {item.get('provider', 'unknown')}")
            if 'date' in item:
                print(f"   日期: {item['date']}")
    else:
        print(f"\n搜索失败: {result.message}")


def example_web_crawl_tool():
    """使用 WebCrawlTool 的示例"""
    print("\n" + "=" * 60)
    print("示例 3: 使用 WebCrawlTool (集成 Metaso)")
    print("=" * 60)
    
    # 创建 mock agent 和 args
    mock_agent = Mock()
    mock_args = Mock()
    mock_args.metaso_api_key = os.getenv('METASO_API_KEY')
    mock_args.firecrawl_api_key = os.getenv('FIRECRAWL_API_KEY')
    
    if not mock_args.metaso_api_key:
        print("\n⚠️  请先设置环境变量 METASO_API_KEY")
        return
    
    # 示例 1: 单页读取
    print("\n1. 单页读取:")
    tool = WebCrawlTool(
        url="https://www.python.org",
        limit=1,
        provider="metaso"
    )
    
    resolver = WebCrawlToolResolver(mock_agent, tool, mock_args)
    result = resolver.resolve()
    
    if result.success:
        print(f"   {result.message}")
        if result.content:
            page = result.content[0]
            print(f"   URL: {page['url']}")
            print(f"   内容长度: {len(page['content'])} 字符")
            print(f"   内容预览: {page['content'][:200]}...")
    else:
        print(f"   失败: {result.message}")
    
    # 示例 2: 多页爬取
    print("\n2. 多页爬取:")
    tool = WebCrawlTool(
        url="https://docs.python.org",
        limit=3,
        include_paths="/tutorial",
        provider="metaso"
    )
    
    resolver = WebCrawlToolResolver(mock_agent, tool, mock_args)
    result = resolver.resolve()
    
    if result.success:
        print(f"   {result.message}")
        for idx, page in enumerate(result.content, 1):
            print(f"\n   页面 {idx}:")
            print(f"     URL: {page['url']}")
            print(f"     标题: {page.get('title', 'N/A')}")
            print(f"     内容长度: {len(page.get('content', ''))} 字符")
    else:
        print(f"   失败: {result.message}")


def example_auto_provider_selection():
    """演示提供商自动选择"""
    print("\n" + "=" * 60)
    print("示例 4: 提供商自动选择")
    print("=" * 60)
    
    mock_agent = Mock()
    mock_args = Mock()
    
    # 设置环境变量中的 API keys
    mock_args.metaso_api_key = os.getenv('METASO_API_KEY')
    mock_args.firecrawl_api_key = os.getenv('FIRECRAWL_API_KEY')
    
    print("\n环境配置:")
    print(f"  Metaso API Key: {'已设置' if mock_args.metaso_api_key else '未设置'}")
    print(f"  Firecrawl API Key: {'已设置' if mock_args.firecrawl_api_key else '未设置'}")
    
    if not (mock_args.metaso_api_key or mock_args.firecrawl_api_key):
        print("\n⚠️  请至少设置一个 API key")
        return
    
    # 不指定 provider，让系统自动选择
    tool = WebSearchTool(
        query="自动选择测试",
        limit=2
        # 注意：没有指定 provider 参数
    )
    
    resolver = WebSearchToolResolver(mock_agent, tool, mock_args)
    
    # 系统会根据可用的 API keys 自动选择：
    # 1. 如果只有 Metaso key -> 使用 Metaso
    # 2. 如果只有 Firecrawl key -> 使用 Firecrawl  
    # 3. 如果两个都有 -> 默认使用 Firecrawl
    
    result = resolver.resolve()
    
    if result.success:
        print(f"\n自动选择结果: {result.message}")
        if "Metaso" in result.message:
            print("  ✓ 系统选择了 Metaso")
        elif "Firecrawl" in result.message:
            print("  ✓ 系统选择了 Firecrawl")
    else:
        print(f"\n执行失败: {result.message}")


def main():
    """运行所有示例"""
    print("\n" + "=" * 60)
    print("Metaso 集成使用示例")
    print("=" * 60)
    
    print("\n使用说明:")
    print("1. 设置 API Key:")
    print("   export METASO_API_KEY='your-metaso-api-key'")
    print("   export FIRECRAWL_API_KEY='your-firecrawl-api-key' (可选)")
    print("\n2. 在工具中指定 provider 参数:")
    print("   provider='metaso' - 使用 Metaso")
    print("   provider='firecrawl' - 使用 Firecrawl")
    print("   provider=None - 自动选择")
    
    # 检查环境变量
    has_metaso = bool(os.getenv('METASO_API_KEY'))
    has_firecrawl = bool(os.getenv('FIRECRAWL_API_KEY'))
    
    print("\n当前环境:")
    print(f"  Metaso API Key: {'✓ 已设置' if has_metaso else '✗ 未设置'}")
    print(f"  Firecrawl API Key: {'✓ 已设置' if has_firecrawl else '✗ 未设置'}")
    
    if not has_metaso:
        print("\n⚠️  请先设置 METASO_API_KEY 环境变量来运行示例")
        print("   export METASO_API_KEY='mk-YOUR_API_KEY_HERE'")
        return
    
    try:
        # 运行示例
        example_direct_metaso_sdk()
        example_web_search_tool()
        example_web_crawl_tool()
        example_auto_provider_selection()
        
        print("\n" + "=" * 60)
        print("✅ 所有示例运行完成！")
        print("=" * 60)
        
    except Exception as e:
        print(f"\n❌ 示例运行失败: {str(e)}")
        import traceback
        traceback.print_exc()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
专门测试消息ID裁剪逻辑的验证脚本

只测试 _apply_message_ids_pruning 方法，不受 token 阈值限制影响

支持的消息ID格式：
1. 直接字段格式: {"message_id": "9226b3a4-1234-5678-9abc-def012345678"}
2. Hint嵌入格式: {"content": "消息内容 [[message_id: 9226b3a4]]"}
3. 兼容格式: {"content": "消息内容 message_id: 9226b3a4"}
"""

import sys
import os
from datetime import datetime

# 添加src路径到Python路径
sys.path.insert(0, os.path.join(os.path.dirname(os.path.abspath(__file__)), '../../../..'))

from autocoder.common.pruner.agentic_conversation_pruner import AgenticConversationPruner
from autocoder.common.pruner.conversation_message_ids_api import get_conversation_message_ids_api
from autocoder.common import AutoCoderArgs
from autocoder.sdk import get_llm


def test_message_ids_pruning_logic():
    """直接测试消息ID裁剪逻辑"""
    
    print("🧪 消息ID裁剪逻辑专项测试")
    print(f"📅 测试时间: {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}")
    print("=" * 80)
    
    # 使用已知的会话ID
    conversation_id = "796e1c2e-1ab8-46d1-9448-f9bd29d5c095"
    
    # 创建测试数据，包含要删除的消息ID
    # 展示两种消息ID格式的支持
    test_conversations = [
        {
            "role": "system",
            "content": "You are a helpful assistant.",
            "message_id": "12345678abcdef01"  # 格式1: 直接字段格式（完整UUID）
        },
        {
            "role": "user", 
            "content": "Hello, can you help me with some code? [[message_id: 87654321]]",  # 格式2: 标准hint格式
            "message_id": "87654321fedcba02"  # 同时测试优先级（字段优先）
        },
        {
            "role": "assistant",
            "content": "Of course! I'd be happy to help you with your code. What do you need assistance with?",
            "message_id": "642f04ee11223344"  # 格式1: 这个消息ID应该被删除
        },
        {
            "role": "user",
            "content": "I want to read a file and analyze it. message_id: 13579bdf",  # 格式2: 兼容hint格式
            # 没有message_id字段，只从content中提取
        },
        {
            "role": "assistant", 
            "content": "I'll help you read and analyze the file. [[message_id: 97531eca]]",  # 格式2: 标准hint格式
            # 没有message_id字段，只从content中提取
        }
    ]
    
    print("📋 测试数据:")
    for i, conv in enumerate(test_conversations, 1):
        role = conv.get("role", "unknown")
        message_id = conv.get("message_id", "no_id")[:8]
        content = conv.get("content", "")[:50] + "..." if len(conv.get("content", "")) > 50 else conv.get("content", "")
        print(f"  {i}. [{role:9}] {message_id} | {content}")
    
    # 查询消息ID配置
    api = get_conversation_message_ids_api()
    message_ids_config = api.get_conversation_message_ids(conversation_id)
    
    if not message_ids_config:
        print(f"❌ 未找到会话ID {conversation_id} 的消息删除配置")
        return False
    
    print(f"\n✅ 消息ID配置:")
    print(f"   要删除的消息ID: {message_ids_config.message_ids}")
    print(f"   保持成对删除: {message_ids_config.preserve_pairs}")
    
    # 创建裁剪器
    args = AutoCoderArgs(
        source_dir=".",
        conversation_prune_safe_zone_tokens=1000,
        context_prune=True,
        context_prune_strategy="extract",
        query="测试消息ID裁剪逻辑"
    )
    llm = get_llm("v3_chat", product_mode="lite")
    
    # 创建裁剪器
    from autocoder.common.conversations.get_conversation_manager import get_conversation_manager
    
    # 设置当前对话ID
    conversation_manager = get_conversation_manager()
    conversation_manager.set_current_conversation(conversation_id)
    
    # 创建AgenticConversationPruner实例并执行测试
    pruner = AgenticConversationPruner(args=args, llm=llm, conversation_id=conversation_id)
    
    # 直接测试消息ID裁剪逻辑
    print(f"\n🚀 执行消息ID裁剪:")
    
    try:
        # 由于 _apply_message_ids_pruning 是私有方法，我们通过完整流程测试但强制低token阈值
        # 设置一个极小的token阈值来强制触发裁剪逻辑
        args.conversation_prune_safe_zone_tokens = 1  # 极小值，确保触发
        
        # 使用完整的 prune_conversations 方法
        pruned_conversations = pruner.prune_conversations(test_conversations)
        
        print(f"\n📊 裁剪结果:")
        print(f"   原始消息数量: {len(test_conversations)}")
        print(f"   裁剪后数量: {len(pruned_conversations)}")
        print(f"   删除的消息数: {len(test_conversations) - len(pruned_conversations)}")
        
        # 检查具体被删除的消息
        original_ids = [conv.get("message_id", "")[:8] for conv in test_conversations]
        pruned_ids = [conv.get("message_id", "")[:8] for conv in pruned_conversations]
        deleted_ids = [msg_id for msg_id in original_ids if msg_id not in pruned_ids]
        remaining_ids = pruned_ids
        
        print(f"\n🔍 详细分析:")
        print(f"   原始消息ID: {original_ids}")
        print(f"   删除的消息ID: {deleted_ids}")
        print(f"   保留的消息ID: {remaining_ids}")
        
        # 验证目标消息ID是否被删除
        target_id = "642f04ee"
        if target_id in deleted_ids:
            print(f"   ✅ 目标消息ID {target_id} 已成功删除")
        else:
            print(f"   ❌ 目标消息ID {target_id} 未被删除")
            return False
        
        # 验证成对删除逻辑
        if message_ids_config.preserve_pairs:
            print(f"\n🔗 成对删除验证:")
            if "87654321" in deleted_ids and "642f04ee" in deleted_ids:
                print(f"   ✅ 成对删除正确：user消息 87654321 和 assistant消息 642f04ee 都被删除")
            else:
                print(f"   ⚠️ 成对删除检查：删除的消息ID = {deleted_ids}")
        
        # 显示裁剪后的消息列表
        print(f"\n📋 裁剪后的消息:")
        for i, conv in enumerate(pruned_conversations, 1):
            role = conv.get("role", "unknown")
            message_id = conv.get("message_id", "no_id")[:8]
            content = conv.get("content", "")[:50] + "..." if len(conv.get("content", "")) > 50 else conv.get("content", "")
            print(f"  {i}. [{role:9}] {message_id} | {content}")
        
        # 获取裁剪统计信息
        stats = pruner.get_pruning_statistics()
        if stats["range_pruning"]["applied"]:
            print(f"\n📈 裁剪统计:")
            print(f"   消息ID裁剪已应用: {stats['range_pruning']['applied']}")
            print(f"   消息ID裁剪成功: {stats['range_pruning']['success']}")
            print(f"   总压缩比: {stats['compression']['total_compression_ratio']:.2%}")
        
        return True
        
    except Exception as e:
        print(f"❌ 消息ID裁剪测试失败: {str(e)}")
        import traceback
        traceback.print_exc()
        return False


def main():
    """主函数"""
    success = test_message_ids_pruning_logic()
    
    print(f"\n🎯 测试结果: {'✅ 通过' if success else '❌ 失败'}")
    
    if success:
        print("\n🎉 消息ID裁剪逻辑工作正常！")
        print("   - 消息ID匹配正确")
        print("   - 成对删除逻辑有效")
        print("   - 裁剪算法运行正常")
    else:
        print("\n⚠️ 消息ID裁剪逻辑存在问题，请检查相关代码")
    
    return success


if __name__ == "__main__":
    success = main()
    sys.exit(0 if success else 1) 


"""
Hook manager for configuration loading, caching, and event processing.
"""

import asyncio
import json
import os
import re
import time
import yaml
from typing import Dict, List, Optional, Tuple

from loguru import logger
from autocoder.common.async_utils import AsyncSyncMixin
from ..agent_events.types import EventMessage
from .types import HooksConfig, HookMatcher, HookProcessingResult
from .hook_executor import HookExecutor


class HookManager(AsyncSyncMixin):
    """
    Core hook management and event processing.
    
    Provides both async and sync methods for all operations.
    Sync methods are automatically generated with a '_sync' suffix.
    """
    
    def __init__(self, config_path: Optional[str] = None, cwd: Optional[str] = None, 
                 command_timeout: int = 30000):
        """
        Initialize the hook manager.
        
        Args:
            config_path: Path to hooks configuration file (supports .json, .yaml, .yml formats)
            cwd: Working directory for command execution
            command_timeout: Command timeout in milliseconds
        """        
        self.cwd = cwd or os.getcwd()
        self.config_path = config_path or self._find_default_config_path()
        self.command_timeout = command_timeout
        
        self._config: Optional[HooksConfig] = None
        self._config_mtime: Optional[float] = None
        self._executor = HookExecutor(cwd=self.cwd, timeout=command_timeout)
        self._config_lock = asyncio.Lock()
    
    def _find_default_config_path(self) -> str:
        """
        Find the default configuration file by checking for hooks.json, hooks.yaml, hooks.yml.
        
        Returns:
            Path to the first found configuration file, defaults to hooks.json if none found
        """
        base_dir = os.path.join(self.cwd, '.auto-coder')
        
        # Priority order: .json first for backward compatibility, then .yaml, then .yml
        candidates = [
            os.path.join(base_dir, 'hooks.json'),
            os.path.join(base_dir, 'hooks.yaml'),
            os.path.join(base_dir, 'hooks.yml')
        ]
        
        for candidate in candidates:
            if os.path.exists(candidate):
                return candidate
        
        # Default to hooks.json for backward compatibility
        return candidates[0]
    
    def _get_config_format(self, config_path: str) -> str:
        """
        Detect configuration file format based on file extension.
        
        Args:
            config_path: Path to configuration file
            
        Returns:
            File format: 'json', 'yaml', or 'unknown'
        """
        ext = os.path.splitext(config_path)[1].lower()
        if ext == '.json':
            return 'json'
        elif ext in ['.yaml', '.yml']:
            return 'yaml'
        else:
            return 'unknown'
    
    def config_exists(self) -> bool:
        """Check if configuration file exists."""
        return os.path.exists(self.config_path)
    
    async def load_config(self) -> Dict[str, any]:
        """
        Load and validate hooks configuration from file.
        Supports both JSON and YAML formats.
        
        Returns:
            Dictionary with 'config' (HooksConfig or None) and 'errors' (list of error messages)
        """
        async with self._config_lock:
            errors = []
            config = None
            
            try:
                if not self.config_exists():
                    errors.append(f"Configuration file not found: {self.config_path}")
                    return {'config': None, 'errors': errors}
                
                # Check if we need to reload based on file modification time
                current_mtime = os.path.getmtime(self.config_path)
                if self._config is not None and self._config_mtime == current_mtime:
                    return {'config': self._config, 'errors': []}
                
                # Detect file format and load accordingly
                config_format = self._get_config_format(self.config_path)
                
                with open(self.config_path, 'r', encoding='utf-8') as f:
                    if config_format == 'json':
                        data = json.load(f)
                    elif config_format == 'yaml':
                        data = yaml.safe_load(f)
                    else:
                        errors.append(f"Unsupported configuration file format: {self.config_path}")
                        return {'config': None, 'errors': errors}
                
                # Validate configuration structure
                validation_errors = self._validate_config(data)
                if validation_errors:
                    errors.extend(validation_errors)
                    return {'config': None, 'errors': errors}
                
                # Create config object
                config = HooksConfig.from_dict(data)
                
                # Cache the configuration
                self._config = config
                self._config_mtime = current_mtime
                
                logger.info(f"Loaded hooks configuration from {self.config_path} (format: {config_format})")
                
            except json.JSONDecodeError as e:
                errors.append(f"Invalid JSON in configuration file: {e}")
            except yaml.YAMLError as e:
                errors.append(f"Invalid YAML in configuration file: {e}")
            except Exception as e:
                errors.append(f"Error loading configuration: {e}")
                logger.error(f"Failed to load hooks configuration: {e}")
            
            return {'config': config, 'errors': errors}
    
    def _validate_config(self, data: Dict) -> List[str]:
        """Validate configuration structure."""
        errors = []
        
        if not isinstance(data, dict):
            errors.append("Configuration must be a JSON object")
            return errors
        
        if 'hooks' not in data:
            errors.append("Configuration must contain 'hooks' key")
            return errors
        
        hooks = data['hooks']
        if not isinstance(hooks, dict):
            errors.append("'hooks' must be an object")
            return errors
        
        # Validate each event type configuration
        for event_type, matchers in hooks.items():
            if not isinstance(matchers, list):
                errors.append(f"Event type '{event_type}' must have a list of matchers")
                continue
            
            for i, matcher in enumerate(matchers):
                if not isinstance(matcher, dict):
                    errors.append(f"Matcher {i} in '{event_type}' must be an object")
                    continue
                
                if 'matcher' not in matcher:
                    errors.append(f"Matcher {i} in '{event_type}' must have 'matcher' field")
                
                if 'hooks' not in matcher:
                    errors.append(f"Matcher {i} in '{event_type}' must have 'hooks' field")
                    continue
                
                hooks_list = matcher['hooks']
                if not isinstance(hooks_list, list):
                    errors.append(f"'hooks' in matcher {i} of '{event_type}' must be a list")
                    continue
                
                for j, hook in enumerate(hooks_list):
                    if not isinstance(hook, dict):
                        errors.append(f"Hook {j} in matcher {i} of '{event_type}' must be an object")
                        continue
                    
                    if 'type' not in hook:
                        errors.append(f"Hook {j} in matcher {i} of '{event_type}' must have 'type' field")
                    
                    if 'command' not in hook:
                        errors.append(f"Hook {j} in matcher {i} of '{event_type}' must have 'command' field")
        
        return errors
    
    async def process_event(self, event_message: EventMessage) -> HookProcessingResult:
        """
        Process an EventMessage and execute matching hooks.
        
        Args:
            event_message: The event to process
            
        Returns:
            HookProcessingResult containing execution results and any errors
        """
        start_time = time.time()
        result = HookProcessingResult(matched=False)
        
        try:
            # Load configuration
            load_result = await self.load_config()
            if load_result['errors']:
                result.errors.extend(load_result['errors'])
                return result
            
            config = load_result['config']
            if not config:
                result.errors.append("No configuration available")
                return result
            
            # Extract event type and tool name
            event_type = event_message.event_type.value
            tool_name = event_message.content.get('tool_name', '')
            
            # Find matching event type configuration
            if event_type not in config.hooks:
                logger.debug(f"No hooks configured for event type: {event_type}")
                return result  # No configuration for this event type
            
            matchers = config.hooks[event_type]
            
            # Process each matcher
            for matcher in matchers:
                try:
                    # Check if tool name matches the pattern
                    if re.search(matcher.matcher, tool_name):
                        result.matched = True
                        logger.info(f"Matched hook pattern '{matcher.matcher}' for tool '{tool_name}'")
                        
                        # Execute hooks for this matcher
                        execution_results = await self._executor.execute_hooks(matcher.hooks, event_message)
                        result.results.extend(execution_results)
                        
                        # Log execution summary
                        successful = sum(1 for r in execution_results if r.success)
                        failed = len(execution_results) - successful
                        logger.info(f"Executed {len(execution_results)} hooks: {successful} successful, {failed} failed")
                        
                except re.error as e:
                    error_msg = f"Invalid regex pattern '{matcher.matcher}': {e}"
                    result.errors.append(error_msg)
                    logger.error(error_msg)
                except Exception as e:
                    error_msg = f"Error processing matcher '{matcher.matcher}': {e}"
                    result.errors.append(error_msg)
                    logger.error(error_msg)
        
        except Exception as e:
            error_msg = f"Error processing event: {e}"
            result.errors.append(error_msg)
            logger.error(error_msg)
        
        finally:
            result.processing_time = time.time() - start_time
            
            if result.matched:
                logger.info(f"Processed event {event_message.event_type.value} in {result.processing_time:.3f}s")
        
        return result
    
    async def reload_config(self) -> Dict[str, any]:
        """Force reload configuration from file."""
        async with self._config_lock:
            self._config = None
            self._config_mtime = None
        return await self.load_config()
    
    def get_current_config(self) -> Optional[HooksConfig]:
        """Get the currently loaded configuration."""
        return self._config 


"""
PersistConversationManager 异常类定义
"""


class ConversationManagerError(Exception):
    """对话管理器基础异常类"""
    
    def __init__(self, message="对话管理器发生错误", error_code="GENERAL_ERROR"):
        super().__init__(message)
        self.error_code = error_code


class ConversationNotFoundError(ConversationManagerError):
    """对话不存在异常"""
    
    def __init__(self, conversation_id):
        # 检查是否是类似ID的字符串（不包含中文等）
        if (isinstance(conversation_id, str) and len(conversation_id) > 0 and 
            not any(ord(c) > 127 or c.isspace() for c in conversation_id)):
            message = f"对话未找到: {conversation_id}"
        else:
            message = conversation_id  # 自定义消息
        super().__init__(message, error_code="CONVERSATION_NOT_FOUND")


class MessageNotFoundError(ConversationManagerError):
    """消息不存在异常"""
    
    def __init__(self, message_id):
        # 检查是否是类似ID的字符串（不包含中文等）
        if (isinstance(message_id, str) and len(message_id) > 0 and 
            not any(ord(c) > 127 or c.isspace() for c in message_id)):
            message = f"消息未找到: {message_id}"
        else:
            message = message_id  # 自定义消息
        super().__init__(message, error_code="MESSAGE_NOT_FOUND")


class ConcurrencyError(ConversationManagerError):
    """并发访问异常"""
    
    def __init__(self, message="并发访问冲突"):
        super().__init__(message, error_code="CONCURRENCY_ERROR")


class DataIntegrityError(ConversationManagerError):
    """数据完整性异常"""
    
    def __init__(self, message="数据完整性检查失败"):
        super().__init__(message, error_code="DATA_INTEGRITY_ERROR")


class LockTimeoutError(ConversationManagerError):
    """锁超时异常"""
    
    def __init__(self, message="锁获取超时"):
        super().__init__(message, error_code="LOCK_TIMEOUT_ERROR")


class BackupError(ConversationManagerError):
    """备份操作异常"""
    
    def __init__(self, message="备份操作失败"):
        super().__init__(message, error_code="BACKUP_ERROR")


class RestoreError(ConversationManagerError):
    """恢复操作异常"""
    
    def __init__(self, message="恢复操作失败"):
        super().__init__(message, error_code="RESTORE_ERROR") 


"""
Token Helper Plugin for Chat Auto Coder.
Provides token counting functionality for files and projects.
"""

import os
from typing import Any, Callable, Dict, List, Optional, Tuple
from dataclasses import dataclass
from collections import defaultdict

from autocoder.plugins import Plugin, PluginManager
from autocoder.common.tokens import count_string_tokens as count_tokens
from autocoder.suffixproject import SuffixProject
from autocoder.common import AutoCoderArgs, SourceCode
from autocoder.common.international import get_message, get_message_with_format
from loguru import logger
import json


@dataclass
class TokenCount:
    """Represents token count information for a file."""
    filename: str
    tokens: int
    relative_path: Optional[str] = None
    file_size: Optional[int] = None


class TokenHelperPlugin(Plugin):
    """Token helper plugin for the Chat Auto Coder."""

    name = "token_helper"
    description = "Token helper plugin providing token counting for files and projects"
    version = "0.1.0"

    def __init__(self, manager: PluginManager, config: Optional[Dict[str, Any]] = None, config_path: Optional[str] = None):
        """Initialize the Token helper plugin."""
        super().__init__(manager, config, config_path)
        self.token_counts = {}
        self.project_dir = os.getcwd()        
        self.base_persist_dir = os.path.join(".auto-coder", "plugins", "chat-auto-coder")        
        self.auto_coder_config = {}
        self.exclude_files = []

    def load_auto_coder_config(self):
        memory_path = os.path.join(self.base_persist_dir, "memory.json")
        if os.path.exists(memory_path):
            with open(memory_path, "r", encoding="utf-8") as f:
                _memory = json.load(f)
                self.auto_coder_config = _memory.get("conf",{})   
                self.exclude_files = _memory.get("exclude_files",[])

    def initialize(self) -> bool:
        """Initialize the plugin.

        Returns:
            True if initialization was successful
        """
        self.load_auto_coder_config()
        print(f"[{self.name}] {get_message('plugin_token_initialized')}")
        return True

    def get_commands(self) -> Dict[str, Tuple[Callable, str]]:
        """Get commands provided by this plugin.

        Returns:
            A dictionary of command name to handler and description
        """
        return {
            "token/count": (self.count_tokens_in_project, get_message("plugin_token_cmd_count_desc")),
            "token/top": (self.show_top_token_files, get_message("plugin_token_cmd_top_desc")),
            "token/file": (self.count_tokens_in_file, get_message("plugin_token_cmd_file_desc")),
            "token/summary": (self.show_token_summary, get_message("plugin_token_cmd_summary_desc")),
        }

    def get_completions(self) -> Dict[str, List[str]]:
        """Get completions provided by this plugin.

        Returns:
            A dictionary mapping command prefixes to completion options
        """
        completions = {
            "/token/count": [],
            "/token/top": ["5", "10", "20", "50", "100"],
            "/token/file": [],
            "/token/summary": [],
        }
        return completions

    def count_tokens_in_project(self, args: str) -> None:
        """Count tokens in all project files.
        
        Args:
            args: Optional arguments including project directory and file types
        """
        args_dict = self._parse_args(args)
        project_dir = args_dict.get("dir", self.auto_coder_config.get("project_dir", os.getcwd()))
        project_type = args_dict.get("type", self.auto_coder_config.get("project_type", ".py"))
        exclude = args_dict.get("exclude_files", [])
        
        self.project_dir = project_dir
        print(get_message_with_format("plugin_token_counting_project", project_dir=project_dir))
        print(get_message_with_format("plugin_token_file_types", project_type=project_type))
        
        try:
            # Create AutoCoderArgs with necessary parameters
            coder_args = AutoCoderArgs(
                source_dir=project_dir,
                project_type=project_type,
                exclude_files=[f"regex://{exclude}"] + self.exclude_files if exclude else self.exclude_files
            )
            
            # Use SuffixProject to get all source files
            project = SuffixProject(args=coder_args)
            
            total_tokens = 0
            file_count = 0
            self.token_counts = {}
            
            print(get_message("plugin_token_scanning_files"))
            
            for source_code in project.get_source_codes():
                file_count += 1
                if file_count % 10 == 0:
                    print(get_message_with_format("plugin_token_processed_files", file_count=file_count))
                
                tokens = count_tokens(source_code.source_code)
                file_path = source_code.module_name
                relative_path = os.path.relpath(file_path, project_dir)
                file_size = len(source_code.source_code)
                
                self.token_counts[file_path] = TokenCount(
                    filename=file_path,
                    tokens=tokens,
                    relative_path=relative_path,
                    file_size=file_size
                )
                
                total_tokens += tokens
            
            print(f"\n{get_message('plugin_token_count_complete')}")
            print(get_message_with_format("plugin_token_total_files", file_count=file_count))
            print(get_message_with_format("plugin_token_total_tokens", total_tokens=f"{total_tokens:,}"))
            print(get_message("plugin_token_use_top_help"))
            print(get_message("plugin_token_use_summary_help"))
            
        except Exception as e:
            logger.error(f"Error counting tokens in project: {str(e)}")
            print(get_message_with_format("plugin_token_counting_error", error=str(e)))

    def _parse_args(self, args: str) -> Dict[str, str]:
        """Parse command arguments.
        
        Args:
            args: Command arguments string. Supports both:
                 - Key=value format: dir=. type=.py,.java
                 - Command line format: --dir . --type .py,.java
            
        Returns:
            Dictionary of parsed arguments
        """
        result = {}
        if not args:
            return result
            
        # Try using argparse first
        try:
            import argparse
            import shlex
            
            # Create parser with arguments expected by the plugin
            parser = argparse.ArgumentParser(description='Token counter options')
            parser.add_argument('--dir', '-d', help='Project directory')
            parser.add_argument('--type', '-t', help='File types (comma separated)')
            parser.add_argument('--exclude', '-e', help='Exclude pattern')
            parser.add_argument('--verbose', '-v', action='store_true', help='Verbose output')
            
            # Parse with shlex to handle quoted arguments properly
            parsed_args, unknown = parser.parse_known_args(shlex.split(args))
            
            # Convert namespace to dictionary, ignoring None values
            for key, value in vars(parsed_args).items():
                if value is not None:
                    result[key] = value
                    
            # Handle any unknown arguments as key=value pairs
            for arg in unknown:
                if '=' in arg:
                    key, value = arg.split('=', 1)
                    # Strip leading dashes if present
                    if key.startswith('--'):
                        key = key[2:]
                    elif key.startswith('-'):
                        key = key[1:]
                    result[key] = value
            
            return result
            
        except Exception as e:
            # Fallback to original implementation if argparse fails
            logger.debug(f"Argparse failed, using fallback parser: {str(e)}")
            
            parts = args.split()
            i = 0
            while i < len(parts):
                part = parts[i]
                
                # Handle key=value format
                if "=" in part:
                    key, value = part.split("=", 1)
                    # Strip leading dashes if present
                    if key.startswith("--"):
                        key = key[2:]
                    elif key.startswith("-"):
                        key = key[1:]
                    result[key] = value
                    i += 1
                    continue
                    
                # Handle --key value or -key value format
                if part.startswith("--"):
                    key = part[2:]
                    # Check if there's a value following this key
                    if i + 1 < len(parts) and not parts[i + 1].startswith("-"):
                        result[key] = parts[i + 1]
                        i += 2
                    else:
                        # Flag option without value
                        result[key] = "true"
                        i += 1
                elif part.startswith("-"):
                    key = part[1:]
                    # Check if there's a value following this key
                    if i + 1 < len(parts) and not parts[i + 1].startswith("-"):
                        result[key] = parts[i + 1]
                        i += 2
                    else:
                        # Flag option without value
                        result[key] = "true"
                        i += 1
                else:
                    # Standalone argument without a key
                    i += 1
            
            return result

    def show_top_token_files(self, args: str) -> None:
        """Show top N files by token count.
        
        Args:
            args: Number of files to show
        """
        if not self.token_counts:
            print(get_message("plugin_token_no_data"))
            return
            
        try:
            n = int(args.strip()) if args.strip() else 10
        except ValueError:
            print(get_message_with_format("plugin_token_invalid_value_default", args=args))
            n = 10
            
        print(f"\n{get_message_with_format('plugin_token_top_files_header', n=n)}")
        
        tokens_header = get_message("plugin_token_table_header_tokens")
        size_header = get_message("plugin_token_table_header_size_bytes") 
        file_header = get_message("plugin_token_table_header_file")
        
        print(f"{tokens_header:<10} {size_header:<15} {file_header}")
        print(f"{'-'*10} {'-'*15} {'-'*50}")
        
        sorted_files = sorted(
            self.token_counts.values(), 
            key=lambda x: x.tokens, 
            reverse=True
        )
        
        for i, token_count in enumerate(sorted_files[:n], 1):
            relative_path = token_count.relative_path or token_count.filename
            print(f"{token_count.tokens:<10,} {token_count.file_size:<15,} {relative_path}")

    def count_tokens_in_file(self, args: str) -> None:
        """Count tokens in a specific file or directory.
        
        Args:
            args: Path to the file or directory. If starts with @, remove @ and treat as path.
        """
        if not args:
            print(get_message("plugin_token_specify_path"))
            return
            
        # Handle @ prefix - remove it and treat as path
        path = args.strip()
        if path.startswith('@'):
            path = path[1:]
        
        if not os.path.exists(path):
            print(get_message_with_format("plugin_token_path_not_exist", path=path))
            return
            
        try:
            if os.path.isfile(path):
                # Handle single file
                self._count_tokens_single_file(path)
            elif os.path.isdir(path):
                # Handle directory recursively
                self._count_tokens_directory(path)
            else:
                print(get_message_with_format("plugin_token_not_file_or_dir", path=path))
                
        except Exception as e:
            print(get_message_with_format("plugin_token_counting_error", error=str(e)))
    
    def _count_tokens_single_file(self, file_path: str) -> int:
        """Count tokens in a single file and display results.
        
        Args:
            file_path: Path to the file
            
        Returns:
            Number of tokens in the file
        """
        try:
            with open(file_path, 'r', encoding='utf-8') as f:
                content = f.read()
                
            tokens = count_tokens(content)
            print(f"\n{get_message_with_format('plugin_token_file_info', file_path=file_path)}")
            print(get_message_with_format("plugin_token_file_tokens", tokens=f"{tokens:,}"))
            print(get_message_with_format("plugin_token_file_size", size=f"{len(content):,}"))
            if tokens > 0:
                avg_bytes = len(content)/tokens
                print(get_message_with_format("plugin_token_avg_bytes_per_token", avg=f"{avg_bytes:.2f}"))
            
            return tokens
            
        except UnicodeDecodeError:
            print(get_message_with_format("plugin_token_skip_binary", file_path=file_path))
            return 0
        except Exception as e:
            print(get_message_with_format("plugin_token_read_error", file_path=file_path, error=str(e)))
            return 0
    
    def _count_tokens_directory(self, dir_path: str) -> None:
        """Count tokens in all files within a directory recursively.
        
        Args:
            dir_path: Path to the directory
        """
        total_tokens = 0
        file_count = 0
        processed_files = []
        
        print(f"\n{get_message_with_format('plugin_token_scanning_directory', dir_path=dir_path)}")
        
        for root, dirs, files in os.walk(dir_path):
            # Skip common ignore directories
            dirs[:] = [d for d in dirs if not d.startswith('.') and d not in ['__pycache__', 'node_modules', 'dist', 'build']]
            
            for file in files:
                # Skip hidden files and common binary/generated files
                if file.startswith('.') or file.endswith(('.pyc', '.pyo', '.so', '.dll', '.exe', '.bin')):
                    continue
                    
                file_path = os.path.join(root, file)
                relative_path = os.path.relpath(file_path, dir_path)
                
                try:
                    with open(file_path, 'r', encoding='utf-8') as f:
                        content = f.read()
                        
                    tokens = count_tokens(content)
                    total_tokens += tokens
                    file_count += 1
                    
                    processed_files.append({
                        'path': relative_path,
                        'tokens': tokens,
                        'size': len(content)
                    })
                    
                except (UnicodeDecodeError, PermissionError):
                    # Skip binary files and files without permission
                    continue
                except Exception as e:
                    print(get_message_with_format("plugin_token_processing_error", relative_path=relative_path, error=str(e)))
                    continue
        
        # Display results
        print(f"\n{get_message('plugin_token_scan_complete')}")
        print(get_message_with_format("plugin_token_files_processed", file_count=file_count))
        print(get_message_with_format("plugin_token_total_tokens", total_tokens=f"{total_tokens:,}"))
        
        if file_count > 0:
            avg_tokens = total_tokens / file_count
            print(get_message_with_format("plugin_token_avg_tokens_per_file", avg_tokens=f"{avg_tokens:.2f}"))
            
            # Show top 10 files by token count
            if len(processed_files) > 1:
                print(f"\n{get_message('plugin_token_top_files_by_count')}")
                sorted_files = sorted(processed_files, key=lambda x: x['tokens'], reverse=True)
                
                tokens_header = get_message("plugin_token_table_header_tokens")
                size_header = get_message("plugin_token_table_header_size")
                file_header = get_message("plugin_token_table_header_file")
                
                print(f"{tokens_header:>8} {size_header:>8} {file_header}")
                print(f"{'-'*8} {'-'*8} {'-'*50}")
                
                for file_info in sorted_files[:10]:
                    print(f"{file_info['tokens']:>8,} {file_info['size']:>8} {file_info['path']}")

    def show_token_summary(self, args: str) -> None:
        """Show token count summary by file type.
        
        Args:
            args: Optional arguments
        """
        if not self.token_counts:
            print(get_message("plugin_token_no_data"))
            return
            
        by_extension = defaultdict(lambda: {"files": 0, "tokens": 0, "size": 0})
        
        for token_count in self.token_counts.values():
            filename = token_count.filename
            ext = os.path.splitext(filename)[1].lower() or "no_extension"
            
            by_extension[ext]["files"] += 1
            by_extension[ext]["tokens"] += token_count.tokens
            by_extension[ext]["size"] += token_count.file_size or 0
            
        total_tokens = sum(data["tokens"] for data in by_extension.values())
        total_files = sum(data["files"] for data in by_extension.values())
        total_size = sum(data["size"] for data in by_extension.values())
        
        print(f"\n{get_message('plugin_token_summary_header')}")
        
        extension_header = get_message("plugin_token_table_header_extension")
        files_header = get_message("plugin_token_table_header_files")
        tokens_header = get_message("plugin_token_table_header_tokens")
        percent_header = get_message("plugin_token_table_header_percent")
        size_kb_header = get_message("plugin_token_table_header_size_kb")
        
        print(f"{extension_header:<12} {files_header:<8} {tokens_header:<12} {percent_header:<12} {size_kb_header:<12}")
        print(f"{'-'*12} {'-'*8} {'-'*12} {'-'*12} {'-'*12}")
        
        for ext, data in sorted(by_extension.items(), key=lambda x: x[1]["tokens"], reverse=True):
            percent = (data["tokens"] / total_tokens * 100) if total_tokens > 0 else 0
            size_kb = data["size"] / 1024
            print(f"{ext:<12} {data['files']:<8} {data['tokens']:<12,} {percent:<12.2f} {size_kb:<12.2f}")
            
        print(f"\n{get_message_with_format('plugin_token_total_files_summary', total_files=f'{total_files:,}')}")
        print(get_message_with_format("plugin_token_total_tokens_summary", total_tokens=f"{total_tokens:,}"))
        print(get_message_with_format("plugin_token_total_size", total_size=f"{total_size/1024/1024:.2f}"))
        
        if self.project_dir:
            print(get_message_with_format("plugin_token_project_directory", project_dir=self.project_dir))

    def shutdown(self) -> None:
        """Shutdown the plugin."""
        print(f"[{self.name}] {get_message('plugin_token_shutdown')}") 


"""
CLI 主入口点模块

提供命令行接口的主要实现，处理参数解析和命令执行。
"""

import sys
import os
import argparse
from typing import Optional, List
from .options import CLIOptions, get_default_async_workdir
from ..models.responses import CLIResult
from .handlers import PrintModeHandler
from autocoder.run_context import get_run_context, RunMode
from autocoder.common.international import get_message, get_message_with_format
from autocoder.common.file_monitor import FileMonitor
import os

# 设置运行模式为终端模式
get_run_context().set_mode(RunMode.CLI)


class AutoCoderCLI:
    """命令行接口主类，处理命令行参数和执行相应的操作。"""

    def __init__(self):
        """初始化CLI实例。"""
        pass

    def run(self, options: CLIOptions, cwd: Optional[str] = None) -> CLIResult:
        """
        运行CLI命令。

        Args:
            options: CLI选项
            cwd: 当前工作目录，如果为None则使用系统当前目录

        Returns:
            命令执行结果
        """
        try:
            # 验证选项
            options.validate()

            # 根据命令类型分发处理
            if options.command == "config":
                return self.handle_config_command(options, cwd)
            else:
                # 默认处理代码生成命令
                return self.handle_print_mode(options, cwd)

        except Exception as e:
            return CLIResult(success=False, output="", error=str(e))

    def handle_config_command(
        self, options: CLIOptions, cwd: Optional[str] = None
    ) -> CLIResult:
        """
        处理 config 命令。

        Args:
            options: CLI选项
            cwd: 当前工作目录

        Returns:
            命令执行结果
        """
        try:
            from autocoder.auto_coder_runner import configure

            if not options.config_args:
                return CLIResult(
                    success=False,
                    output="",
                    error="配置参数不能为空。用法: auto-coder.run config model=v3_chat",
                )

            # 处理每个配置参数
            results = []
            for config_arg in options.config_args:
                if "=" not in config_arg:
                    return CLIResult(
                        success=False,
                        output="",
                        error=f"配置参数格式错误: {config_arg}。应使用 key=value 格式",
                    )

                # 将 key=value 转换为 key:value 格式（configure 函数需要的格式）
                key, value = config_arg.split("=", 1)
                config_str = f"{key.strip()}:{value.strip()}"

                try:
                    configure(config_str, skip_print=True)
                    results.append(f"✓ 配置成功: {key.strip()} = {value.strip()}")
                except Exception as e:
                    return CLIResult(
                        success=False,
                        output="",
                        error=f"配置失败 {config_str}: {str(e)}",
                    )

            output = "\n".join(results)
            return CLIResult(success=True, output=output)

        except Exception as e:
            return CLIResult(
                success=False,
                output="",
                error=get_message_with_format("config_command_failed", error=str(e)),
            )

    def handle_print_mode(
        self, options: CLIOptions, cwd: Optional[str] = None
    ) -> CLIResult:
        """
        处理打印模式。

        Args:
            options: CLI选项
            cwd: 当前工作目录

        Returns:
            命令执行结果
        """
        # 检查是否启用异步模式
        if options.async_mode:
            from ..async_runner.async_handler import AsyncAgentHandler

            handler = AsyncAgentHandler(options, cwd)
            return handler.handle()
        else:
            handler = PrintModeHandler(options, cwd)
            return handler.handle()

    @classmethod
    def parse_args(cls, args: Optional[List[str]] = None) -> CLIOptions:
        """
        解析命令行参数。

        Args:
            args: 命令行参数列表，如果为None则使用sys.argv

        Returns:
            解析后的CLI选项
        """
        parser = argparse.ArgumentParser(
            description=get_message("cli_description"),
            formatter_class=argparse.RawDescriptionHelpFormatter,
            epilog=get_message("cli_examples"),
        )

        # 创建子命令解析器
        subparsers = parser.add_subparsers(
            dest="command", help=get_message("help_available_commands")
        )

        # config 子命令
        config_parser = subparsers.add_parser(
            "config", help=get_message("help_config_command")
        )
        config_parser.add_argument(
            "config_args", nargs="*", help=get_message("help_config_args")
        )

        # 默认命令（代码生成）- 当没有子命令时使用
        # 会话选项（移除模式选择，--continue 和 --resume 作为独立参数）
        parser.add_argument(
            "-c",
            "--continue",
            dest="continue_session",
            action="store_true",
            help=get_message("help_continue_session"),
        )
        parser.add_argument(
            "-r",
            "--resume",
            dest="resume_session",
            metavar="SESSION_ID",
            help=get_message("help_resume_session"),
        )

        # 输入输出选项
        parser.add_argument("prompt", nargs="?", help=get_message("help_prompt"))
        parser.add_argument(
            "--output-format",
            choices=["text", "json", "stream-json"],
            default="text",
            help=get_message("help_output_format"),
        )
        parser.add_argument(
            "--input-format",
            choices=["text", "json", "stream-json"],
            default="text",
            help=get_message("help_input_format"),
        )
        parser.add_argument(
            "-v", "--verbose", action="store_true", help=get_message("help_verbose")
        )

        # 高级选项
        advanced = parser.add_argument_group(get_message("help_advanced_options"))
        advanced.add_argument(
            "--max-turns", type=int, default=10000, help=get_message("help_max_turns")
        )
        advanced.add_argument("--system-prompt", help=get_message("help_system_prompt"))
        advanced.add_argument(
            "--system-prompt-path", help=get_message("help_system_prompt_path")
        )
        advanced.add_argument(
            "--allowed-tools", nargs="+", help=get_message("help_allowed_tools")
        )
        advanced.add_argument(
            "--permission-mode",
            choices=["manual", "acceptEdits"],
            default="manual",
            help=get_message("help_permission_mode"),
        )
        advanced.add_argument("--model", required=False, help=get_message("help_model"))
        advanced.add_argument(
            "--include-rules",
            action="store_true",
            help=get_message("help_include_rules"),
        )
        advanced.add_argument("--pr", action="store_true", help=get_message("help_pr"))
        advanced.add_argument(
            "--is-sub-agent", action="store_true", help=get_message("help_is_sub_agent")
        )
        advanced.add_argument(
            "--loop", type=int, default=1, help=get_message("help_loop")
        )
        advanced.add_argument(
            "--loop-keep-conversation",
            dest="loop_keep_conversation",
            action="store_true",
            help=get_message("help_loop_keep_conversation"),
        )
        advanced.add_argument(
            "--loop-additional-prompt",
            dest="loop_additional_prompt",
            help=get_message("help_loop_additional_prompt"),
        )
        advanced.add_argument(
            "--include-libs", dest="include_libs", help=get_message("help_include_libs")
        )
        advanced.add_argument(
            "--workflow", help="执行指定的 workflow（名称或文件路径）"
        )

        # 异步代理运行器选项
        async_group = parser.add_argument_group(get_message("help_async_group_title"))
        async_group.add_argument(
            "--async",
            dest="async_mode",
            action="store_true",
            help=get_message("help_async_mode"),
        )
        async_group.add_argument(
            "--split",
            dest="split_mode",
            choices=["h1", "h2", "h3", "any", "delimiter"],
            default="h1",
            help=get_message("help_split_mode"),
        )
        async_group.add_argument(
            "--delimiter", default="===", help=get_message("help_delimiter")
        )
        async_group.add_argument(
            "--min-level", type=int, default=1, help=get_message("help_min_level")
        )
        async_group.add_argument(
            "--max-level", type=int, default=3, help=get_message("help_max_level")
        )
        async_group.add_argument(
            "--workdir",
            default=get_default_async_workdir(),
            help=get_message("help_workdir"),
        )
        async_group.add_argument(
            "--from",
            dest="from_branch",
            default="",
            help=get_message("help_from_branch"),
        )
        async_group.add_argument(
            "--bg",
            dest="bg_mode",
            action="store_true",
            help=get_message("help_bg_mode"),
        )
        async_group.add_argument(
            "--task-prefix",
            dest="task_prefix",
            default="",
            help=get_message("help_task_prefix"),
        )
        async_group.add_argument(
            "--worktree-name",
            dest="worktree_name",
            help=get_message("help_worktree_name"),
        )

        # 解析参数
        parsed_args = parser.parse_args(args)

        # 转换为CLIOptions
        options = CLIOptions(
            command=getattr(parsed_args, "command", None),
            config_args=getattr(parsed_args, "config_args", []),
            continue_session=parsed_args.continue_session,
            resume_session=parsed_args.resume_session,
            prompt=parsed_args.prompt,
            output_format=parsed_args.output_format,
            input_format=parsed_args.input_format,
            verbose=parsed_args.verbose,
            max_turns=parsed_args.max_turns,
            system_prompt=parsed_args.system_prompt,
            system_prompt_path=getattr(parsed_args, "system_prompt_path", None),
            allowed_tools=parsed_args.allowed_tools or [],
            permission_mode=parsed_args.permission_mode,
            model=parsed_args.model,
            include_rules=parsed_args.include_rules,
            pr=parsed_args.pr,
            is_sub_agent=parsed_args.is_sub_agent,
            loop=parsed_args.loop,
            loop_keep_conversation=parsed_args.loop_keep_conversation,
            loop_additional_prompt=parsed_args.loop_additional_prompt,
            include_libs=(
                [lib.strip() for lib in parsed_args.include_libs.split(",")]
                if parsed_args.include_libs
                else []
            ),
            workflow=getattr(parsed_args, "workflow", None),
            # 异步代理运行器相关参数
            async_mode=parsed_args.async_mode,
            split_mode=parsed_args.split_mode,
            delimiter=parsed_args.delimiter,
            min_level=parsed_args.min_level,
            max_level=parsed_args.max_level,
            workdir=parsed_args.workdir,
            from_branch=parsed_args.from_branch,
            bg_mode=parsed_args.bg_mode,
            task_prefix=parsed_args.task_prefix,
            worktree_name=parsed_args.worktree_name,
        )

        return options

    @classmethod
    def main(cls) -> int:
        """
        CLI主入口点。

        Returns:
            退出码，0表示成功，非0表示失败
        """
        try:
            options = cls.parse_args()
            cli = cls()
            result = cli.run(options)

            if result.success:
                if result.output:
                    print(result.output)
                return 0
            else:
                print(
                    get_message_with_format("error_prefix", error=result.error),
                    file=sys.stderr,
                )
                return 1

        except Exception as e:
            # 如果是 verbose 模式，打印完整的异常堆栈
            if hasattr(options, "verbose") and options.verbose:
                import traceback

                print(f"[DEBUG] Unhandled exception:", file=sys.stderr)
                traceback.print_exc(file=sys.stderr)
            print(
                get_message_with_format("unhandled_error", error=str(e)),
                file=sys.stderr,
            )
            return 1

        finally:
            try:
                FileMonitor(os.getcwd()).stop()
            except Exception as e:
                pass


if __name__ == "__main__":
    sys.exit(AutoCoderCLI.main())



"""
Event manager for handling events between System A and System B.
"""

from loguru import logger
import os
import time
import threading
from typing import Optional, List, Dict, Any, Callable, Union, TypeVar, Generic

from .event_store import EventStore, JsonlEventStore
from .event_types import Event, EventType, ResponseEvent

T = TypeVar("T")


class EventManager:
    """
    Manager for handling events between two systems.
    System A writes events and can be blocked by special events.
    System B reads events and can respond to special events.
    """

    def __init__(self, event_store: EventStore):
        """
        Initialize an event manager.

        Args:
            event_store: The event store to use
        """
        self.event_store = event_store
        self._last_read_event_id: Optional[str] = self.event_store._last_event_id
        self._blocking_events: Dict[str, threading.Event] = {}
        self._response_callbacks: Dict[str, Callable[[str], None]] = {}

    @classmethod
    def create(cls, file_path: str) -> "EventManager":
        """
        Create an event manager with a JSONL event store.

        Args:
            file_path: Path to the JSONL file

        Returns:
            An initialized event manager
        """
        store = JsonlEventStore(file_path)
        return cls(store)

    def write_result(
        self, content: Union[Dict[str, Any], Any], metadata: Dict[str, Any] = {}
    ) -> Event:
        """
        Write a result event.

        Args:
            content: The content of the result
            metadata: The metadata of the result
        Returns:
            The created event
        """
        if not isinstance(content, dict):
            content = content.to_dict()
        event = Event(event_type=EventType.RESULT, content=content, metadata=metadata)
        self.event_store.append_event(event)
        return event

    def write_completion(
        self, content: Union[Dict[str, Any], Any], metadata: Dict[str, Any] = {}
    ) -> Event:
        """
        Write a completion event.

        Args:
            content: The content of the completion (CompletionContent or dict)

        Returns:
            The created event
        """
        if not isinstance(content, dict):
            content = content.to_dict()
        event = Event(
            event_type=EventType.COMPLETION, content=content, metadata=metadata
        )
        self.event_store.append_event(event)
        return event

    def write_error(
        self, content: Union[Dict[str, Any], Any], metadata: Dict[str, Any] = {}
    ) -> Event:
        """
        Write an error event.

        Args:
            content: The content of the error (ErrorContent or dict)

        Returns:
            The created event
        """
        if not isinstance(content, dict):
            content = content.to_dict()
        event = Event(event_type=EventType.ERROR, content=content, metadata=metadata)
        self.event_store.append_event(event)
        return event

    def write_stream(
        self, content: Union[Dict[str, Any], Any], metadata: Dict[str, Any] = {}
    ) -> Event:
        """
        Write a stream event.

        Args:
            content: The content of the stream
            metadata: The metadata of the stream
        Returns:
            The created event
        """
        if not isinstance(content, dict):
            content = content.to_dict()
        event = Event(event_type=EventType.STREAM, content=content, metadata=metadata)
        self.event_store.append_event(event)
        return event

    def ask_user(
        self,
        prompt: str,
        options: Optional[List[str]] = None,
        callback: Optional[Callable[[str], None]] = None,
        timeout: Optional[float] = None,
    ) -> str:
        """
        Ask the user a question and wait for response.
        This method blocks until a response is received or timeout expires.

        Args:
            prompt: The question to ask
            options: Optional list of valid responses
            callback: Optional callback function that will be called with the response
            timeout: Optional timeout in seconds. None means use default (3600s).

        Returns:
            The user's response, or empty string on timeout
        """
        logger.debug(f"ask_user called with prompt: {prompt}, timeout: {timeout}")
        content: Dict[str, Any] = {"prompt": prompt}
        if options:
            content["options"] = options
        if timeout is not None:
            content["timeout"] = timeout

        # 实际用于 blocker.wait 的超时：调用者指定则使用，否则默认 3600 秒
        effective_timeout: float = timeout if timeout is not None else 60.0 * 60

        event = Event(event_type=EventType.ASK_USER, content=content)
        logger.debug(f"Created ASK_USER event with ID: {event.event_id}")

        # Register a blocking event
        blocker = threading.Event()
        self.event_store.append_event(event)
        self._blocking_events[event.event_id] = blocker
        logger.info(
            f"ASK_USER: {self} {self.event_store.file_path} self._blocking_events: {self._blocking_events}"
        )
        if callback:
            self._response_callbacks[event.event_id] = callback
            logger.debug(f"Registered callback for event: {event.event_id}")

        # Wait for response
        logger.debug(
            f"Waiting for response to event: {event.event_id}, effective_timeout: {effective_timeout}s"
        )

        wait_result = blocker.wait(timeout=effective_timeout)

        if not wait_result:
            logger.warning(
                f"Timeout ({effective_timeout}s) waiting for response to event: {event.event_id}"
            )
            # 超时后清理 blocking event，避免资源泄漏
            if event.event_id in self._blocking_events:
                del self._blocking_events[event.event_id]
            if event.event_id in self._response_callbacks:
                del self._response_callbacks[event.event_id]
            return ""

        logger.debug(
            f"Received signal that response is available for: {event.event_id}"
        )

        # Get the response event
        def is_response(e: Event) -> bool:
            return (
                isinstance(e, ResponseEvent)
                and e.event_type == EventType.USER_RESPONSE
                and e.response_to == event.event_id
            )

        response_event = self.event_store.wait_for_event(is_response, timeout=1.0)

        # 获取响应内容
        response = ""
        if response_event and "response" in response_event.content:
            response = response_event.content["response"]
            logger.debug(
                f"Retrieved response: '{response}' from event: {response_event.event_id}"
            )
        else:
            logger.warning("No valid response found in event store")

        # 清理资源（移到这里确保在获取响应后才清理）
        if event.event_id in self._blocking_events:
            del self._blocking_events[event.event_id]
            logger.debug(f"Clean up blocker for event: {event.event_id}")
        else:
            logger.debug(f"No blocker found for event: {event.event_id} during cleanup")

            # 注意：回调不在这里清理，而是在respond_to_user中清理
            # 这样回调函数仍然可用

        return response

    def respond_to_user(self, ask_event_id: str, response: str) -> ResponseEvent:
        """
        Respond to a user question.

        Args:
            ask_event_id: ID of the ASK_USER event
            response: The response to provide

        Returns:
            The created response event
        """
        logger.info(
            f"respond_to_user called for event: {ask_event_id} with response: '{response}'"
        )

        # 创建响应事件
        event = ResponseEvent(
            event_type=EventType.USER_RESPONSE,
            content={"response": response},
            response_to=ask_event_id,
        )

        # 存储响应事件
        self.event_store.append_event(event)
        logger.info(f"Response event created and stored with ID: {event.event_id}")

        # 获取回调和阻塞器
        callback = None

        # 检查是否有回调
        if ask_event_id in self._response_callbacks:
            callback = self._response_callbacks[ask_event_id]
            # 暂时不删除回调，确保不会丢失
            logger.info(f"Retrieved callback for event: {ask_event_id}")
        else:
            logger.info(f"No callback found for event: {ask_event_id}")

        # 如果找到了回调，执行它
        if callback:
            try:
                logger.info(f"Executing callback for event: {ask_event_id}")
                callback(response)
                logger.info(f"Callback execution completed for event: {ask_event_id}")

                if ask_event_id in self._response_callbacks:
                    del self._response_callbacks[ask_event_id]
            except Exception as e:
                logger.error(f"Error in response callback: {e}")

        # 检查是否存在对应的阻塞事件
        logger.info(
            f"RESPONSD_TO_USER: {self} {self.event_store.file_path} self._blocking_events: {self._blocking_events}"
        )
        if ask_event_id in self._blocking_events:
            self._blocking_events[ask_event_id].set()
            logger.info(f"Unblocked event: {ask_event_id}")
        else:
            logger.warning(f"No blocking event found for event_id: {ask_event_id}")

        return event

    def read_events(
        self,
        event_types: Optional[List[EventType]] = None,
        block: bool = False,
        timeout: Optional[float] = None,
    ) -> List[Event]:
        """
        Read events from the store.

        Args:
            event_types: Only get events of these types
            block: Whether to block until new events are available
            timeout: Maximum time to wait if blocking

        Returns:
            List of events
        """
        events = self.event_store.get_events(
            after_id=self._last_read_event_id, event_types=event_types
        )

        if events:
            self._last_read_event_id = events[-1].event_id
            return events

        # No events and not blocking
        if not block:
            return []

        # Wait for new events
        def is_new_event(e: Event) -> bool:
            if event_types and e.event_type not in event_types:
                return False

            return (
                self._last_read_event_id is None
                or e.event_id != self._last_read_event_id
            )

        event = self.event_store.wait_for_event(is_new_event, timeout=timeout)

        if event:
            events = self.event_store.get_events(
                after_id=self._last_read_event_id, event_types=event_types
            )

            if events:
                self._last_read_event_id = events[-1].event_id
                return events

        return []

    def close(self) -> None:
        """
        Close the event manager and its event store.
        """
        # Unblock any waiting threads
        for blocker in self._blocking_events.values():
            blocker.set()

        # Close the event store
        if isinstance(self.event_store, JsonlEventStore):
            self.event_store.close()

    def truncate(self) -> None:
        """
        Clear all events from the event store.
        This truncates the underlying file and resets the last read event ID.

        Warning: This operation cannot be undone and will permanently delete all events.
        """
        if not hasattr(self.event_store, "truncate"):
            raise NotImplementedError(
                "The current event store does not support truncation"
            )

        # Reset the last read event ID
        self._last_read_event_id = None

        # Clear blocking events and callbacks
        self._blocking_events.clear()
        self._response_callbacks.clear()

        # Truncate the store
        self.event_store.truncate()

        logger.debug("Event store has been truncated")



from enum import Enum
import pydantic
from typing import List, Dict, Tuple,Any,Optional
class Mode(Enum):
    MULTI_ROUND = "multi_round"
    SINGLE_ROUND = "single_round"

class StepNum(pydantic.BaseModel):
    step_num:int= pydantic.Field(1,description="总共步骤数")
    content:int= pydantic.Field(1,description="详细的执行步骤，每个步骤需要包含一个shell/python 代码块")    

class CodeGenerateResult(pydantic.BaseModel):
    contents:List[str]
    conversations:List[List[Dict[str, Any]]]
    metadata:Dict[str, Any] = {}

class MergeCodeWithoutEffect(pydantic.BaseModel):
    success_blocks: List[Tuple[str, str]]
    failed_blocks: List[Any]
    merged_blocks: Optional[Any] = None


"""
CLI 选项定义模块

定义命令行接口的选项数据结构。
"""

import os
from pathlib import Path
from dataclasses import dataclass, field
from typing import Optional, Dict, Any

from ..constants import DEFAULT_MODEL
from ..models.responses import CLIResult  # 从 models.responses 导入，避免重复定义
from autocoder.common.international import get_message_with_format, get_message


def get_default_async_workdir() -> str:
    """获取默认的异步运行器工作目录，支持多平台"""
    home_dir = Path.home()
    return str(home_dir / ".auto-coder" / "async_agent")


@dataclass
class CLIOptions:
    """CLI选项数据类，用于配置命令行工具的行为。"""

    # 命令选项
    command: Optional[str] = None  # 子命令名称，如 'config'
    config_args: list = field(default_factory=list)  # config 命令的参数列表

    # 会话选项
    continue_session: bool = False  # 继续最近的对话
    resume_session: Optional[str] = None  # 恢复特定会话的ID

    # 输入选项
    prompt: Optional[str] = None  # 提示内容，如果为None则从stdin读取
    input_format: str = "text"  # 输入格式，可选值: text, json, stream-json

    # 输出选项
    output_format: str = "text"  # 输出格式，可选值: text, json, stream-json
    verbose: bool = False  # 是否输出详细信息

    # 高级选项
    max_turns: int = 10000  # 最大对话轮数
    system_prompt: Optional[str] = None  # 系统提示
    system_prompt_path: Optional[str] = None  # 系统提示文件路径
    allowed_tools: list = field(default_factory=list)  # 允许使用的工具列表
    permission_mode: str = "manual"  # 权限模式，可选值: manual, acceptEdits
    model: Optional[str] = None  # 模型名称，如 gpt-4, gpt-3.5-turbo 等
    include_rules: bool = False  # 是否包含规则文件
    pr: bool = False  # 是否创建 PR
    is_sub_agent: bool = False  # 是否为子代理模式，避免conversation冲突
    loop: int = 1  # 循环执行次数，默认为1
    loop_keep_conversation: bool = False  # 循环时是否保持对话连续性
    loop_additional_prompt: Optional[str] = None  # 循环时的额外提示，可自定义
    include_libs: list = field(
        default_factory=list
    )  # 要包含的LLM friendly packages列表

    # Workflow 选项
    workflow: Optional[str] = None  # workflow 名称或文件路径（仅非 async 模式支持）

    # Async Agent Runner 选项
    async_mode: bool = False  # 是否启用异步代理运行器模式
    split_mode: str = "h1"  # 分割模式: h1, h2, h3, any, delimiter
    delimiter: str = "==="  # 自定义分隔符（当 split_mode=delimiter 时使用）
    min_level: int = 1  # 最小标题级别（当 split_mode=any 时使用）
    max_level: int = 3  # 最大标题级别（当 split_mode=any 时使用）
    workdir: str = field(default_factory=get_default_async_workdir)  # 工作目录
    from_branch: str = ""  # 基础分支（为空时自动检测）
    bg_mode: bool = False  # 后台运行模式
    task_prefix: str = ""  # 任务名称前缀（异步模式下使用）
    worktree_name: Optional[str] = None  # 指定的 worktree 名称（为空时自动生成）

    def validate(self) -> None:
        """验证选项的有效性。"""
        # 验证输出格式
        valid_formats = ["text", "json", "stream-json"]
        if self.output_format not in valid_formats:
            raise ValueError(
                get_message_with_format(
                    "invalid_output_format",
                    format=self.output_format,
                    valid_formats=", ".join(valid_formats),
                )
            )

        # stream-json 格式建议开启verbose以获得实时输出
        # 但不强制要求，JSON格式可以在非verbose模式下正常工作

        # 验证输入格式
        if self.input_format not in valid_formats:
            raise ValueError(
                get_message_with_format(
                    "invalid_input_format",
                    format=self.input_format,
                    valid_formats=", ".join(valid_formats),
                )
            )

        # 验证权限模式
        valid_permission_modes = ["manual", "acceptEdits"]
        if self.permission_mode not in valid_permission_modes:
            raise ValueError(
                get_message_with_format(
                    "invalid_permission_mode",
                    mode=self.permission_mode,
                    valid_modes=", ".join(valid_permission_modes),
                )
            )

        # 验证会话选项的互斥性
        if self.continue_session and self.resume_session:
            raise ValueError(get_message_with_format("session_options_conflict"))

        # 验证 system_prompt 和 system_prompt_path 的互斥性
        if self.system_prompt and self.system_prompt_path:
            raise ValueError(
                "只能配置 --system-prompt 或 --system-prompt-path 中的一个，不能同时配置两个"
            )

        # 验证异步模式相关参数
        if self.async_mode:
            # 验证分割模式
            valid_split_modes = ["h1", "h2", "h3", "any", "delimiter"]
            if self.split_mode not in valid_split_modes:
                raise ValueError(
                    get_message_with_format(
                        "invalid_split_mode",
                        mode=self.split_mode,
                        valid_modes=", ".join(valid_split_modes),
                    )
                )

            # 验证标题级别范围
            if self.split_mode == "any":
                if self.min_level < 1 or self.max_level < 1:
                    raise ValueError(get_message("heading_level_must_be_positive"))
                if self.min_level > self.max_level:
                    raise ValueError(get_message("min_level_cannot_exceed_max"))

            # 验证分隔符
            if self.split_mode == "delimiter" and not self.delimiter.strip():
                raise ValueError(get_message("delimiter_required_for_delimiter_mode"))

        # 验证 workflow 选项
        if self.workflow:
            # workflow 仅支持 text 输出格式
            if self.output_format != "text":
                raise ValueError("--workflow 仅支持 --output-format text")

        # 自动选择模型或验证指定的模型（config 命令时跳过）
        # 在 workflow 模式下跳过模型自动选择与校验，允许使用 YAML 中的模型配置
        if self.command != "config" and not self.workflow:
            if not self.model or not self.model.strip():
                self.model = self._auto_select_model()
                if not self.model:
                    raise ValueError(
                        get_message_with_format("model_auto_select_failed")
                    )

    def _auto_select_model(self) -> Optional[str]:
        """自动选择第一个设置了API key的模型"""
        try:
            from autocoder.common.llms import LLMManager

            manager = LLMManager()
            all_models = manager.get_all_models()

            # 遍历所有模型，找到第一个有API key的模型
            for model_name, model in all_models.items():
                if manager.has_key(model_name):
                    return model_name

            return None
        except Exception:
            # 如果导入或初始化失败，返回 None
            return None



"""
Type definitions for the Agent Hooks system.
"""

from typing import Dict, List, Any, Optional
from dataclasses import dataclass, field
from enum import Enum
import time

class HookType(str, Enum):
    """Types of hooks supported by the system."""
    COMMAND = "command"

@dataclass
class Hook:
    """Individual hook definition with type and command."""
    type: HookType
    command: str
    description: Optional[str] = None
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert hook to dictionary."""
        result = {
            'type': self.type.value,
            'command': self.command
        }
        if self.description:
            result['description'] = self.description
        return result
    
    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'Hook':
        """Create hook from dictionary."""
        return cls(
            type=HookType(data.get('type', HookType.COMMAND.value)),
            command=data.get('command', ''),
            description=data.get('description')
        )

@dataclass 
class HookMatcher:
    """Pattern matching and hook configuration."""
    matcher: str  # Regular expression pattern
    hooks: List[Hook] = field(default_factory=list)
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert hook matcher to dictionary."""
        return {
            'matcher': self.matcher,
            'hooks': [hook.to_dict() for hook in self.hooks]
        }
    
    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'HookMatcher':
        """Create hook matcher from dictionary."""
        hooks = []
        for hook_data in data.get('hooks', []):
            hooks.append(Hook.from_dict(hook_data))
        
        return cls(
            matcher=data.get('matcher', ''),
            hooks=hooks
        )

@dataclass
class HooksConfig:
    """Complete configuration structure with event type mapping."""
    hooks: Dict[str, List[HookMatcher]] = field(default_factory=dict)
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert hooks config to dictionary."""
        result = {}
        for event_type, matchers in self.hooks.items():
            result[event_type] = [matcher.to_dict() for matcher in matchers]
        return {'hooks': result}
    
    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'HooksConfig':
        """Create hooks config from dictionary."""
        hooks = {}
        hooks_data = data.get('hooks', {})
        
        for event_type, matchers_data in hooks_data.items():
            matchers = []
            for matcher_data in matchers_data:
                matchers.append(HookMatcher.from_dict(matcher_data))
            hooks[event_type] = matchers
        
        return cls(hooks=hooks)

@dataclass
class HookExecutionResult:
    """Command execution results and metrics."""
    success: bool
    command: str
    stdout: str = ""
    stderr: str = ""
    exit_code: int = 0
    execution_time: float = 0.0
    start_time: float = field(default_factory=time.time)
    end_time: Optional[float] = None
    error_message: Optional[str] = None
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert execution result to dictionary."""
        return {
            'success': self.success,
            'command': self.command,
            'stdout': self.stdout,
            'stderr': self.stderr,
            'exit_code': self.exit_code,
            'execution_time': self.execution_time,
            'start_time': self.start_time,
            'end_time': self.end_time,
            'error_message': self.error_message
        }

@dataclass
class HookProcessingResult:
    """Overall event processing results."""
    matched: bool
    results: List[HookExecutionResult] = field(default_factory=list)
    errors: List[str] = field(default_factory=list)
    processing_time: float = 0.0
    
    def to_dict(self) -> Dict[str, Any]:
        """Convert processing result to dictionary."""
        return {
            'matched': self.matched,
            'results': [result.to_dict() for result in self.results],
            'errors': self.errors,
            'processing_time': self.processing_time
        } 


"""AutoCoder Completer Module

This module provides command completion functionality for the AutoCoder CLI.
"""

from .command_completer_v2 import CommandCompleterV2

__all__ = ['CommandCompleterV2'] 


import importlib.resources as resources
from tokenizers import Tokenizer
from typing import Optional

class BuildinTokenizer:
    _instance: Optional['BuildinTokenizer'] = None
    _tokenizer: Optional[Tokenizer] = None

    def __new__(cls) -> 'BuildinTokenizer':
        if cls._instance is None:
            cls._instance = super(BuildinTokenizer, cls).__new__(cls)
            cls._instance._initialize()
        return cls._instance

    def _initialize(self) -> None:
        try:
            tokenizer_path = str(resources.files("autocoder") / "data" / "tokenizer.json")
        except FileNotFoundError:
            tokenizer_path = None
            
        if tokenizer_path:
            self._tokenizer = Tokenizer.from_file(tokenizer_path)
        else:
            raise ValueError("Cannot find tokenizer.json file in package data directory")

    def count_tokens(self, text: str) -> int:
        if not self._tokenizer:
            raise ValueError("Tokenizer is not initialized")
            
        encoded = self._tokenizer.encode('{"role":"user","content":"' + text + '"}')
        return len(encoded.ids)

    @property
    def tokenizer(self) -> Optional[Tokenizer]:
        return self._tokenizer